from collections.abc import Callable, Iterable, Iterator, Mapping
from re import Pattern
from typing import Any, TypeVar, overload

from . import resolver as resolver  # Help mypy a bit; this is implied by loader and dumper
from .constructor import BaseConstructor
from .cyaml import *
from .cyaml import _CLoader
from .dumper import *
from .dumper import _Inf
from .emitter import _WriteStream
from .error import *
from .events import *
from .loader import *
from .loader import _Loader
from .nodes import *
from .reader import _ReadStream
from .representer import BaseRepresenter
from .resolver import BaseResolver
from .tokens import *

_T = TypeVar("_T")
_Constructor = TypeVar("_Constructor", bound=BaseConstructor)
_Representer = TypeVar("_Representer", bound=BaseRepresenter)

__with_libyaml__: bool
__version__: str

def warnings(settings=None): ...
def scan(stream, Loader: type[_Loader | _CLoader] = ...): ...
def parse(stream, Loader: type[_Loader | _CLoader] = ...): ...
def compose(stream, Loader: type[_Loader | _CLoader] = ...): ...
def compose_all(stream, Loader: type[_Loader | _CLoader] = ...): ...
def load(stream: _ReadStream, Loader: type[_Loader | _CLoader]) -> Any: ...
def load_all(stream: _ReadStream, Loader: type[_Loader | _CLoader]) -> Iterator[Any]: ...
def full_load(stream: _ReadStream) -> Any: ...
def full_load_all(stream: _ReadStream) -> Iterator[Any]: ...
def safe_load(stream: _ReadStream) -> Any: ...
def safe_load_all(stream: _ReadStream) -> Iterator[Any]: ...
def unsafe_load(stream: _ReadStream) -> Any: ...
def unsafe_load_all(stream: _ReadStream) -> Iterator[Any]: ...
def emit(
    events,
    stream: _WriteStream[Any] | None = None,
    Dumper=...,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
): ...
@overload
def serialize_all(
    nodes,
    stream: _WriteStream[Any],
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> None: ...
@overload
def serialize_all(
    nodes,
    stream: None = None,
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> str: ...
@overload
def serialize_all(
    nodes,
    stream: None = None,
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> bytes: ...
@overload
def serialize(
    node,
    stream: _WriteStream[Any],
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> None: ...
@overload
def serialize(
    node,
    stream: None = None,
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> str: ...
@overload
def serialize(
    node,
    stream: None = None,
    Dumper=...,
    *,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
) -> bytes: ...
@overload
def dump_all(
    documents: Iterable[Any],
    stream: _WriteStream[Any],
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> None: ...
@overload
def dump_all(
    documents: Iterable[Any],
    stream: None = None,
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> str: ...
@overload
def dump_all(
    documents: Iterable[Any],
    stream: None = None,
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> bytes: ...
@overload
def dump(
    data: Any,
    stream: _WriteStream[Any],
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> None: ...
@overload
def dump(
    data: Any,
    stream: None = None,
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> str: ...
@overload
def dump(
    data: Any,
    stream: None = None,
    Dumper=...,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> bytes: ...
@overload
def safe_dump_all(
    documents: Iterable[Any],
    stream: _WriteStream[Any],
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> None: ...
@overload
def safe_dump_all(
    documents: Iterable[Any],
    stream: None = None,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> str: ...
@overload
def safe_dump_all(
    documents: Iterable[Any],
    stream: None = None,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> bytes: ...
@overload
def safe_dump(
    data: Any,
    stream: _WriteStream[Any],
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str | None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> None: ...
@overload
def safe_dump(
    data: Any,
    stream: None = None,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: None = None,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> str: ...
@overload
def safe_dump(
    data: Any,
    stream: None = None,
    *,
    default_style: str | None = None,
    default_flow_style: bool | None = False,
    canonical: bool | None = None,
    indent: int | None = None,
    width: int | _Inf | None = None,
    allow_unicode: bool | None = None,
    line_break: str | None = None,
    encoding: str,
    explicit_start: bool | None = None,
    explicit_end: bool | None = None,
    version: tuple[int, int] | None = None,
    tags: Mapping[str, str] | None = None,
    sort_keys: bool = True,
) -> bytes: ...
def add_implicit_resolver(
    tag: str,
    regexp: Pattern[str],
    first: Iterable[Any] | None = None,
    Loader: type[BaseResolver] | None = None,
    Dumper: type[BaseResolver] = ...,
) -> None: ...
def add_path_resolver(
    tag: str,
    path: Iterable[Any],
    kind: type[Any] | None = None,
    Loader: type[BaseResolver] | None = None,
    Dumper: type[BaseResolver] = ...,
) -> None: ...
@overload
def add_constructor(
    tag: str, constructor: Callable[[Loader | FullLoader | UnsafeLoader, Node], Any], Loader: None = None
) -> None: ...
@overload
def add_constructor(tag: str, constructor: Callable[[_Constructor, Node], Any], Loader: type[_Constructor]) -> None: ...
@overload
def add_multi_constructor(
    tag_prefix: str, multi_constructor: Callable[[Loader | FullLoader | UnsafeLoader, str, Node], Any], Loader: None = None
) -> None: ...
@overload
def add_multi_constructor(
    tag_prefix: str, multi_constructor: Callable[[_Constructor, str, Node], Any], Loader: type[_Constructor]
) -> None: ...
@overload
def add_representer(data_type: type[_T], representer: Callable[[Dumper, _T], Node]) -> None: ...
@overload
def add_representer(data_type: type[_T], representer: Callable[[_Representer, _T], Node], Dumper: type[_Representer]) -> None: ...
@overload
def add_multi_representer(data_type: type[_T], multi_representer: Callable[[Dumper, _T], Node]) -> None: ...
@overload
def add_multi_representer(
    data_type: type[_T], multi_representer: Callable[[_Representer, _T], Node], Dumper: type[_Representer]
) -> None: ...

class YAMLObjectMetaclass(type):
    def __init__(cls, name, bases, kwds) -> None: ...

class YAMLObject(metaclass=YAMLObjectMetaclass):
    __slots__ = ()
    yaml_loader: Any
    yaml_dumper: Any
    yaml_tag: Any
    yaml_flow_style: Any
    @classmethod
    def from_yaml(cls, loader, node): ...
    @classmethod
    def to_yaml(cls, dumper, data): ...



import datetime
from typing import Final
from typing_extensions import TypeGuard, TypeIs, deprecated

from pytz.tzinfo import BaseTzInfo

from .prop import vText

__all__ = ["UIDGenerator", "is_date", "is_datetime", "to_datetime", "is_pytz", "is_pytz_dt", "normalize_pytz"]

class UIDGenerator:
    chars: Final[list[str]]
    @staticmethod
    @deprecated("Use the Python standard library's :func:`uuid.uuid4` instead.")
    def rnd_string(length: int = 16) -> str: ...
    @staticmethod
    @deprecated("Use the Python standard library's :func:`uuid.uuid5` instead.")
    def uid(host_name: str = "example.com", unique: str = "") -> vText: ...

def is_date(dt: datetime.date) -> bool: ...  # and not datetime.date
def is_datetime(dt: datetime.date) -> TypeIs[datetime.datetime]: ...
def to_datetime(dt: datetime.date) -> datetime.datetime: ...
def is_pytz(tz: datetime.tzinfo) -> TypeIs[BaseTzInfo]: ...
def is_pytz_dt(dt: datetime.date) -> TypeGuard[datetime.datetime]: ...  # and dt.tzinfo is BaseTZInfo
def normalize_pytz(dt: datetime.date) -> datetime.datetime: ...



from _typeshed import Incomplete

from networkx.utils.backends import _dispatchable

from ..classes.graph import Graph

__all__ = ["partial_duplication_graph", "duplication_divergence_graph"]

@_dispatchable
def partial_duplication_graph(N, n, p, q, seed=None): ...
@_dispatchable
def duplication_divergence_graph(n, p, seed=None) -> Graph[Incomplete]: ...



from _typeshed import Incomplete

from . import screen

def DoEmit(fsm) -> None: ...
def DoStartNumber(fsm) -> None: ...
def DoBuildNumber(fsm) -> None: ...
def DoBackOne(fsm) -> None: ...
def DoBack(fsm) -> None: ...
def DoDownOne(fsm) -> None: ...
def DoDown(fsm) -> None: ...
def DoForwardOne(fsm) -> None: ...
def DoForward(fsm) -> None: ...
def DoUpReverse(fsm) -> None: ...
def DoUpOne(fsm) -> None: ...
def DoUp(fsm) -> None: ...
def DoHome(fsm) -> None: ...
def DoHomeOrigin(fsm) -> None: ...
def DoEraseDown(fsm) -> None: ...
def DoErase(fsm) -> None: ...
def DoEraseEndOfLine(fsm) -> None: ...
def DoEraseLine(fsm) -> None: ...
def DoEnableScroll(fsm) -> None: ...
def DoCursorSave(fsm) -> None: ...
def DoCursorRestore(fsm) -> None: ...
def DoScrollRegion(fsm) -> None: ...
def DoMode(fsm) -> None: ...
def DoLog(fsm) -> None: ...

class term(screen.screen):
    def __init__(self, r: int = 24, c: int = 80, *args, **kwargs) -> None: ...

class ANSI(term):
    state: Incomplete
    def __init__(self, r: int = 24, c: int = 80, *args, **kwargs) -> None: ...
    def process(self, c) -> None: ...
    def process_list(self, l) -> None: ...
    def write(self, s) -> None: ...
    def flush(self) -> None: ...
    def write_ch(self, ch) -> None: ...
    def do_sgr(self, fsm) -> None: ...
    def do_decsca(self, fsm) -> None: ...
    def do_modecrap(self, fsm) -> None: ...



from typing_extensions import assert_type


def test_min_builtin() -> None:
    # legal comparisons that succeed at runtime
    b1, b2 = bool(True), bool(False)
    i1, i2 = int(1), int(2)
    s1, s2 = str("a"), str("b")
    f1, f2 = float(0.5), float(2.3)
    l1, l2 = list[int]([1, 2]), list[int]([3, 4])
    t1, t2 = tuple[str, str](("A", "B")), tuple[str, str](("C", "D"))
    tN = tuple[str, ...](["A", "B", "C"])

    assert_type(min(b1, b2), bool)
    assert_type(min(i1, i2), int)
    assert_type(min(s1, s2), str)
    assert_type(min(f1, f2), float)

    # mixed numerical types (note: float = int or float)
    assert_type(min(b1, i1), int)
    assert_type(min(i1, b1), int)

    assert_type(min(b1, f1), float)
    assert_type(min(f1, b1), float)

    assert_type(min(i1, f1), float)
    assert_type(min(f1, i1), float)

    # comparisons with lists and tuples
    assert_type(min(l1, l2), list[int])
    assert_type(min(t1, t2), tuple[str, str])
    assert_type(min(tN, t2), tuple[str, ...])


def test_min_bad_builtin() -> None:
    # illegal comparisons that fail at runtime
    i1 = int(1)
    s1 = str("a")
    f1 = float(1.0)
    c1, c2 = complex(1.0, 2.0), complex(3.0, 4.0)
    list_str = list[str](["A", "B"])
    list_int = list[int]([2, 3])
    tup_str = tuple[str, str](("A", "B"))
    tup_int = tuple[int, int]((2, 3))

    # True negatives.
    min(c1, c2)  # type: ignore

    # FIXME: False negatives.
    min(i1, s1)
    min(s1, f1)
    min(f1, list_str)
    min(list_str, list_int)
    min(tup_str, tup_int)


def test_min_custom_comparison() -> None:
    class BoolScalar:
        def __bool__(self) -> bool: ...

    class FloatScalar:
        def __float__(self) -> float: ...
        def __ge__(self, other: "FloatScalar") -> BoolScalar: ...
        def __gt__(self, other: "FloatScalar") -> BoolScalar: ...
        def __lt__(self, other: "FloatScalar") -> BoolScalar: ...
        def __le__(self, other: "FloatScalar") -> BoolScalar: ...

    f1 = FloatScalar()
    f2 = FloatScalar()

    assert_type(min(f1, f2), FloatScalar)


def test_min_bad_custom_type() -> None:
    class FloatScalar:
        def __float__(self) -> float: ...
        def __ge__(self, other: "FloatScalar") -> object:
            return object()

        def __gt__(self, other: "FloatScalar") -> object:
            return object()

        def __lt__(self, other: "FloatScalar") -> object:
            return object()

        def __le__(self, other: "FloatScalar") -> object:
            return object()

    f1 = FloatScalar()
    f2 = FloatScalar()

    # Note: min(f1, f2) works at runtime, but always returns the second argument.
    #   therefore, we require returning a boolean-like type for comparisons.
    min(f1, f2)  # type: ignore



from collections.abc import Callable, Hashable
from datetime import date
from re import Pattern
from typing import Any, ClassVar, TypeVar

from yaml.error import MarkedYAMLError
from yaml.nodes import MappingNode, Node, ScalarNode, SequenceNode

from .cyaml import _CLoader
from .loader import _Loader

_L = TypeVar("_L", bound=_Loader | _CLoader)
_N = TypeVar("_N", bound=Node)

class ConstructorError(MarkedYAMLError): ...

class BaseConstructor:
    yaml_constructors: Any
    yaml_multi_constructors: Any
    constructed_objects: Any
    recursive_objects: Any
    state_generators: Any
    deep_construct: Any
    def __init__(self) -> None: ...
    def check_data(self): ...
    def check_state_key(self, key: str) -> None: ...
    def get_data(self): ...
    def get_single_data(self) -> Any: ...
    def construct_document(self, node): ...
    def construct_object(self, node, deep=False): ...
    def construct_scalar(self, node: ScalarNode) -> str: ...
    def construct_sequence(self, node: SequenceNode, deep: bool = False) -> list[Any]: ...
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]: ...
    def construct_pairs(self, node, deep=False): ...
    @classmethod
    # Use typevars so we can have covariant behaviour in the parameter types
    def add_constructor(cls, tag: str, constructor: Callable[[_L, _N], Any]) -> None: ...
    @classmethod
    def add_multi_constructor(cls, tag_prefix, multi_constructor): ...

class SafeConstructor(BaseConstructor):
    def construct_scalar(self, node: ScalarNode | MappingNode) -> str: ...
    def flatten_mapping(self, node: MappingNode) -> None: ...
    def construct_mapping(self, node: MappingNode, deep: bool = False) -> dict[Hashable, Any]: ...
    def construct_yaml_null(self, node: ScalarNode) -> None: ...
    bool_values: ClassVar[dict[str, bool]]
    def construct_yaml_bool(self, node: ScalarNode) -> bool: ...
    def construct_yaml_int(self, node: ScalarNode) -> int: ...
    inf_value: ClassVar[float]
    nan_value: ClassVar[float]
    def construct_yaml_float(self, node: ScalarNode) -> float: ...
    def construct_yaml_binary(self, node: ScalarNode) -> bytes: ...
    timestamp_regexp: ClassVar[Pattern[str]]
    def construct_yaml_timestamp(self, node: ScalarNode) -> date: ...
    def construct_yaml_omap(self, node): ...
    def construct_yaml_pairs(self, node): ...
    def construct_yaml_set(self, node): ...
    def construct_yaml_str(self, node): ...
    def construct_yaml_seq(self, node): ...
    def construct_yaml_map(self, node): ...
    def construct_yaml_object(self, node, cls): ...
    def construct_undefined(self, node): ...

class FullConstructor(SafeConstructor):
    def get_state_keys_blacklist(self) -> list[str]: ...
    def get_state_keys_blacklist_regexp(self) -> Pattern[str]: ...
    def construct_python_str(self, node): ...
    def construct_python_unicode(self, node): ...
    def construct_python_bytes(self, node): ...
    def construct_python_long(self, node): ...
    def construct_python_complex(self, node): ...
    def construct_python_tuple(self, node): ...
    def find_python_module(self, name: str, mark, unsafe: bool = False): ...
    def find_python_name(self, name: str, mark, unsafe: bool = False): ...
    def construct_python_name(self, suffix, node): ...
    def construct_python_module(self, suffix, node): ...
    def make_python_instance(self, suffix, node, args=None, kwds=None, newobj: bool = False, unsafe: bool = False): ...
    def set_python_instance_state(self, instance: Any, state, unsafe: bool = False) -> None: ...
    def construct_python_object(self, suffix, node): ...
    def construct_python_object_apply(self, suffix, node, newobj=False): ...
    def construct_python_object_new(self, suffix, node): ...

class UnsafeConstructor(FullConstructor):
    def find_python_module(self, name: str, mark): ...  # type: ignore[override]
    def find_python_name(self, name: str, mark): ...  # type: ignore[override]
    def make_python_instance(self, suffix: str, node, args=None, kwds=None, newobj: bool = False): ...  # type: ignore[override]
    def set_python_instance_state(self, instance: Any, state): ...  # type: ignore[override]

class Constructor(SafeConstructor):
    def construct_python_str(self, node): ...
    def construct_python_unicode(self, node): ...
    def construct_python_long(self, node): ...
    def construct_python_complex(self, node): ...
    def construct_python_tuple(self, node): ...
    def find_python_module(self, name, mark): ...
    def find_python_name(self, name, mark): ...
    def construct_python_name(self, suffix, node): ...
    def construct_python_module(self, suffix, node): ...
    def make_python_instance(self, suffix, node, args=None, kwds=None, newobj=False): ...
    def set_python_instance_state(self, instance, state): ...
    def construct_python_object(self, suffix, node): ...
    def construct_python_object_apply(self, suffix, node, newobj=False): ...
    def construct_python_object_new(self, suffix, node): ...

__all__ = ["BaseConstructor", "SafeConstructor", "FullConstructor", "UnsafeConstructor", "Constructor", "ConstructorError"]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import typing

import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class RecvBufRespExtra(google.protobuf.message.Message):
    """Extra data needed on a non-RDMA RecvBufResponse."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    TENSOR_CONTENT_FIELD_NUMBER: builtins.int
    @property
    def tensor_content(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]: ...
    def __init__(self, *, tensor_content: collections.abc.Iterable[builtins.bytes] | None = ...) -> None: ...
    def ClearField(self, field_name: typing.Literal["tensor_content", b"tensor_content"]) -> None: ...

global___RecvBufRespExtra = RecvBufRespExtra



import sys
from _typeshed import ReadableBuffer, StrPath, SupportsRead, _T_co
from collections.abc import Iterable
from typing import Final, Protocol, type_check_only
from typing_extensions import TypeAlias
from xml.sax._exceptions import (
    SAXException as SAXException,
    SAXNotRecognizedException as SAXNotRecognizedException,
    SAXNotSupportedException as SAXNotSupportedException,
    SAXParseException as SAXParseException,
    SAXReaderNotAvailable as SAXReaderNotAvailable,
)
from xml.sax.handler import ContentHandler as ContentHandler, ErrorHandler as ErrorHandler
from xml.sax.xmlreader import InputSource as InputSource, XMLReader

@type_check_only
class _SupportsReadClose(SupportsRead[_T_co], Protocol[_T_co]):
    def close(self) -> None: ...

_Source: TypeAlias = StrPath | _SupportsReadClose[bytes] | _SupportsReadClose[str]

default_parser_list: Final[list[str]]

def make_parser(parser_list: Iterable[str] = ()) -> XMLReader: ...
def parse(source: _Source, handler: ContentHandler, errorHandler: ErrorHandler = ...) -> None: ...
def parseString(string: ReadableBuffer | str, handler: ContentHandler, errorHandler: ErrorHandler | None = ...) -> None: ...
def _create_parser(parser_name: str) -> XMLReader: ...

if sys.version_info >= (3, 14):
    __all__ = [
        "ContentHandler",
        "ErrorHandler",
        "InputSource",
        "SAXException",
        "SAXNotRecognizedException",
        "SAXNotSupportedException",
        "SAXParseException",
        "SAXReaderNotAvailable",
        "default_parser_list",
        "make_parser",
        "parse",
        "parseString",
    ]



from typing import Final, TypeVar

_CharT = TypeVar("_CharT", str, int)

NUL: Final = 0x00
SOH: Final = 0x01
STX: Final = 0x02
ETX: Final = 0x03
EOT: Final = 0x04
ENQ: Final = 0x05
ACK: Final = 0x06
BEL: Final = 0x07
BS: Final = 0x08
TAB: Final = 0x09
HT: Final = 0x09
LF: Final = 0x0A
NL: Final = 0x0A
VT: Final = 0x0B
FF: Final = 0x0C
CR: Final = 0x0D
SO: Final = 0x0E
SI: Final = 0x0F
DLE: Final = 0x10
DC1: Final = 0x11
DC2: Final = 0x12
DC3: Final = 0x13
DC4: Final = 0x14
NAK: Final = 0x15
SYN: Final = 0x16
ETB: Final = 0x17
CAN: Final = 0x18
EM: Final = 0x19
SUB: Final = 0x1A
ESC: Final = 0x1B
FS: Final = 0x1C
GS: Final = 0x1D
RS: Final = 0x1E
US: Final = 0x1F
SP: Final = 0x20
DEL: Final = 0x7F

controlnames: Final[list[int]]

def isalnum(c: str | int) -> bool: ...
def isalpha(c: str | int) -> bool: ...
def isascii(c: str | int) -> bool: ...
def isblank(c: str | int) -> bool: ...
def iscntrl(c: str | int) -> bool: ...
def isdigit(c: str | int) -> bool: ...
def isgraph(c: str | int) -> bool: ...
def islower(c: str | int) -> bool: ...
def isprint(c: str | int) -> bool: ...
def ispunct(c: str | int) -> bool: ...
def isspace(c: str | int) -> bool: ...
def isupper(c: str | int) -> bool: ...
def isxdigit(c: str | int) -> bool: ...
def isctrl(c: str | int) -> bool: ...
def ismeta(c: str | int) -> bool: ...
def ascii(c: _CharT) -> _CharT: ...
def ctrl(c: _CharT) -> _CharT: ...
def alt(c: _CharT) -> _CharT: ...
def unctrl(c: str | int) -> str: ...



import argparse
import tokenize
from _typeshed import Incomplete
from collections.abc import Sequence
from logging import Logger
from typing import Any
from typing_extensions import TypeAlias

from .plugins.finder import Checkers, LoadedPlugin
from .processor import _LogicalMapping
from .style_guide import StyleGuideManager

Results: TypeAlias = list[tuple[str, int, int, str, str | None]]

LOG: Logger
SERIAL_RETRY_ERRNOS: Incomplete

class Manager:
    style_guide: Incomplete
    options: Incomplete
    plugins: Incomplete
    jobs: Incomplete
    statistics: Incomplete
    exclude: Incomplete
    argv: Incomplete
    results: Incomplete
    def __init__(self, style_guide: StyleGuideManager, plugins: Checkers, argv: Sequence[str]) -> None: ...
    def report(self) -> tuple[int, int]: ...
    def run_parallel(self) -> None: ...
    def run_serial(self) -> None: ...
    def run(self) -> None: ...
    filenames: Incomplete
    def start(self) -> None: ...
    def stop(self) -> None: ...

class FileChecker:
    options: Incomplete
    filename: Incomplete
    plugins: Incomplete
    results: Incomplete
    statistics: Incomplete
    processor: Incomplete
    display_name: Incomplete
    should_process: bool
    def __init__(self, *, filename: str, plugins: Checkers, options: argparse.Namespace) -> None: ...
    def report(self, error_code: str | None, line_number: int, column: int, text: str) -> str: ...
    def run_check(self, plugin: LoadedPlugin, **arguments: Any) -> Any: ...
    def run_ast_checks(self) -> None: ...
    def run_logical_checks(self) -> None: ...
    def run_physical_checks(self, physical_line: str) -> None: ...
    def process_tokens(self) -> None: ...
    def run_checks(self) -> tuple[str, Results, dict[str, int]]: ...
    def handle_newline(self, token_type: int) -> None: ...
    def check_physical_eol(self, token: tokenize.TokenInfo, prev_physical: str) -> None: ...

def find_offset(offset: int, mapping: _LogicalMapping) -> tuple[int, int]: ...



from _typeshed import Incomplete
from collections.abc import Iterable

class EndpointConfig(dict[str, Incomplete]):
    def __init__(
        self,
        version: str,
        aliases: list[str] | None = None,
        links: dict[str, str] | dict[str, None] | dict[str, str | None] | Iterable[tuple[str, str | None]] | None = None,
        ipv4_address: str | None = None,
        ipv6_address: str | None = None,
        link_local_ips: list[str] | None = None,
        driver_opt=None,
        mac_address: str | None = None,
    ) -> None: ...

class NetworkingConfig(dict[str, Incomplete]):
    def __init__(self, endpoints_config: EndpointConfig | None = None) -> None: ...

class IPAMConfig(dict[str, Incomplete]):
    def __init__(
        self,
        driver: str = "default",
        pool_configs: list[IPAMPool] | None = None,
        options: dict[Incomplete, Incomplete] | None = None,
    ) -> None: ...

class IPAMPool(dict[str, Incomplete]):
    def __init__(
        self,
        subnet: str | None = None,
        iprange: str | None = None,
        gateway: str | None = None,
        aux_addresses: dict[str, str] | None = None,
    ) -> None: ...



from collections.abc import Callable

from simplejson.decoder import JSONDecoder

class JSONDecodeError(ValueError):
    msg: str
    doc: str
    pos: int
    end: int | None
    lineno: int
    colno: int
    endlineno: int | None
    endcolno: int | None

def make_scanner(context: JSONDecoder) -> Callable[[str, int], tuple[bool, int]]: ...

__all__ = ["make_scanner", "JSONDecodeError"]



from ._cryptography_key import load_pem_key as load_pem_key
from .asymmetric_key import AsymmetricKey as AsymmetricKey
from .base_key import Key as Key
from .jwk import JsonWebKey as JsonWebKey
from .key_set import KeySet as KeySet

__all__ = ["Key", "AsymmetricKey", "KeySet", "JsonWebKey", "load_pem_key"]



from _typeshed import Incomplete
from collections.abc import Iterable, Sequence
from socket import socket
from typing import Final
from typing_extensions import TypeAlias

# Really complex, consider unpacking a TypedDict
_AdjustmentsParams: TypeAlias = Incomplete

truthy: frozenset[str]
KNOWN_PROXY_HEADERS: Final[frozenset[str]]

def asbool(s: bool | str | int | None) -> bool: ...
def asoctal(s: str) -> int: ...
def aslist_cronly(value: str) -> list[str]: ...
def aslist(value: str) -> list[str]: ...
def asset(value: str | None) -> set[str]: ...
def slash_fixed_str(s: str | None) -> str: ...
def str_iftruthy(s: str | None) -> str | None: ...
def as_socket_list(sockets: Sequence[object]) -> list[socket]: ...

class _str_marker(str): ...
class _int_marker(int): ...

class Adjustments:
    host: _str_marker
    port: _int_marker
    listen: list[str]
    threads: int
    trusted_proxy: str | None
    trusted_proxy_count: int | None
    trusted_proxy_headers: set[str]
    log_untrusted_proxy_headers: bool
    clear_untrusted_proxy_headers: bool
    url_scheme: str
    url_prefix: str
    ident: str
    backlog: int
    recv_bytes: int
    send_bytes: int
    outbuf_overflow: int
    outbuf_high_watermark: int
    inbuf_overflow: int
    connection_limit: int
    cleanup_interval: int
    channel_timeout: int
    log_socket_errors: bool
    max_request_header_size: int
    max_request_body_size: int
    expose_tracebacks: bool
    unix_socket: str | None
    unix_socket_perms: int
    socket_options: list[tuple[int, int, int]]
    asyncore_loop_timeout: int
    asyncore_use_poll: bool
    ipv4: bool
    ipv6: bool
    sockets: list[socket]
    channel_request_lookahead: int
    server_name: str
    def __init__(self, **kw: _AdjustmentsParams) -> None: ...
    @classmethod
    def parse_args(cls, argv: str) -> tuple[dict[str, bool], list[str]]: ...
    @classmethod
    def check_sockets(cls, sockets: Iterable[socket]) -> None: ...



from networkx.classes.digraph import DiGraph
from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from numpy.random import RandomState

__all__ = ["double_edge_swap", "connected_double_edge_swap", "directed_edge_swap"]

@_dispatchable
def directed_edge_swap(G: DiGraph[_Node], *, nswap: int = 1, max_tries: int = 100, seed: int | RandomState | None = None): ...
@_dispatchable
def double_edge_swap(G: Graph[_Node], nswap: int = 1, max_tries: int = 100, seed: int | RandomState | None = None): ...
@_dispatchable
def connected_double_edge_swap(
    G: Graph[_Node], nswap: int = 1, _window_threshold: int = 3, seed: int | RandomState | None = None
) -> int: ...



from setuptools.dist import Distribution

from .._distutils.command import install_scripts as orig

class install_scripts(orig.install_scripts):
    distribution: Distribution  # override distutils.dist.Distribution with setuptools.dist.Distribution
    no_ep: bool
    def initialize_options(self) -> None: ...
    outfiles: list[str]
    def run(self) -> None: ...
    def write_script(self, script_name, contents, mode: str = "t", *ignored) -> None: ...



from _typeshed import Incomplete

from ..rest import RestClientOptions
from ..types import TimeoutType

class Organizations:
    domain: Incomplete
    protocol: Incomplete
    client: Incomplete
    def __init__(
        self,
        domain: str,
        token: str,
        telemetry: bool = True,
        timeout: TimeoutType = 5.0,
        protocol: str = "https",
        rest_options: RestClientOptions | None = None,
    ) -> None: ...
    def all_organizations(
        self,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
    ) -> dict[str, Incomplete]: ...
    async def all_organizations_async(
        self,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
    ) -> dict[str, Incomplete]: ...
    def get_organization_by_name(self, name: str | None = None) -> dict[str, Incomplete]: ...
    async def get_organization_by_name_async(self, name: str | None = None) -> dict[str, Incomplete]: ...
    def get_organization(self, id: str) -> dict[str, Incomplete]: ...
    async def get_organization_async(self, id: str) -> dict[str, Incomplete]: ...
    def create_organization(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_organization_async(self, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def update_organization(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def update_organization_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def delete_organization(self, id: str): ...
    async def delete_organization_async(self, id: str): ...
    def all_organization_connections(
        self, id: str, page: int | None = None, per_page: int | None = None
    ) -> list[dict[str, Incomplete]]: ...
    async def all_organization_connections_async(
        self, id: str, page: int | None = None, per_page: int | None = None
    ) -> list[dict[str, Incomplete]]: ...
    def get_organization_connection(self, id: str, connection_id: str) -> dict[str, Incomplete]: ...
    async def get_organization_connection_async(self, id: str, connection_id: str) -> dict[str, Incomplete]: ...
    def create_organization_connection(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_organization_connection_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def update_organization_connection(
        self, id: str, connection_id: str, body: dict[str, Incomplete]
    ) -> dict[str, Incomplete]: ...
    async def update_organization_connection_async(
        self, id: str, connection_id: str, body: dict[str, Incomplete]
    ) -> dict[str, Incomplete]: ...
    def delete_organization_connection(self, id: str, connection_id: str): ...
    async def delete_organization_connection_async(self, id: str, connection_id: str): ...
    def all_organization_members(
        self,
        id: str,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
        fields: list[str] | None = None,
        include_fields: bool = True,
    ) -> dict[str, Incomplete]: ...
    async def all_organization_members_async(
        self,
        id: str,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = True,
        from_param: str | None = None,
        take: int | None = None,
        fields: list[str] | None = None,
        include_fields: bool = True,
    ) -> dict[str, Incomplete]: ...
    def create_organization_members(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_organization_members_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def delete_organization_members(self, id: str, body: dict[str, Incomplete]): ...
    async def delete_organization_members_async(self, id: str, body: dict[str, Incomplete]): ...
    def all_organization_member_roles(
        self, id: str, user_id: str, page: int | None = None, per_page: int | None = None, include_totals: bool = False
    ) -> list[dict[str, Incomplete]]: ...
    async def all_organization_member_roles_async(
        self, id: str, user_id: str, page: int | None = None, per_page: int | None = None, include_totals: bool = False
    ) -> list[dict[str, Incomplete]]: ...
    def create_organization_member_roles(self, id: str, user_id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_organization_member_roles_async(
        self, id: str, user_id: str, body: dict[str, Incomplete]
    ) -> dict[str, Incomplete]: ...
    def delete_organization_member_roles(self, id: str, user_id: str, body: dict[str, Incomplete]): ...
    async def delete_organization_member_roles_async(self, id: str, user_id: str, body: dict[str, Incomplete]): ...
    def all_organization_invitations(
        self, id: str, page: int | None = None, per_page: int | None = None, include_totals: bool = False
    ) -> dict[str, Incomplete]: ...
    async def all_organization_invitations_async(
        self, id: str, page: int | None = None, per_page: int | None = None, include_totals: bool = False
    ) -> dict[str, Incomplete]: ...
    def get_organization_invitation(self, id: str, invitaton_id: str) -> dict[str, Incomplete]: ...
    async def get_organization_invitation_async(self, id: str, invitaton_id: str) -> dict[str, Incomplete]: ...
    def create_organization_invitation(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    async def create_organization_invitation_async(self, id: str, body: dict[str, Incomplete]) -> dict[str, Incomplete]: ...
    def delete_organization_invitation(self, id: str, invitation_id: str): ...
    async def delete_organization_invitation_async(self, id: str, invitation_id: str): ...
    def get_client_grants(
        self,
        id: str,
        audience: str | None = None,
        client_id: str | None = None,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = False,
    ) -> dict[str, Incomplete]: ...
    async def get_client_grants_async(
        self,
        id: str,
        audience: str | None = None,
        client_id: str | None = None,
        page: int | None = None,
        per_page: int | None = None,
        include_totals: bool = False,
    ) -> dict[str, Incomplete]: ...
    def add_client_grant(self, id: str, grant_id: str) -> dict[str, Incomplete]: ...
    async def add_client_grant_async(self, id: str, grant_id: str) -> dict[str, Incomplete]: ...
    def delete_client_grant(self, id: str, grant_id: str) -> dict[str, Incomplete]: ...
    async def delete_client_grant_async(self, id: str, grant_id: str) -> dict[str, Incomplete]: ...



from _typeshed import StrOrBytesPath, StrPath
from typing import Literal, overload

@overload
def make_archive(
    base_name: str,
    format: str,
    root_dir: StrOrBytesPath | None = None,
    base_dir: str | None = None,
    verbose: bool = False,
    dry_run: bool = False,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
@overload
def make_archive(
    base_name: StrPath,
    format: str,
    root_dir: StrOrBytesPath,
    base_dir: str | None = None,
    verbose: bool = False,
    dry_run: bool = False,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
def make_tarball(
    base_name: str,
    base_dir: StrPath,
    compress: Literal["gzip", "bzip2", "xz"] | None = "gzip",
    verbose: bool = False,
    dry_run: bool = False,
    owner: str | None = None,
    group: str | None = None,
) -> str: ...
def make_zipfile(base_name: str, base_dir: StrPath, verbose: bool = False, dry_run: bool = False) -> str: ...



from _typeshed.wsgi import StartResponse, WSGIApplication, WSGIEnvironment
from abc import abstractmethod
from collections.abc import Iterable
from typing import Any, Literal, TypedDict, type_check_only
from typing_extensions import Unpack

from fanstatic.core import Dependable, NeededResources, Resource
from fanstatic.inclusion import Inclusion
from webob import Request, Response

@type_check_only
class _NeededResourcesConfig(TypedDict, total=False):
    versioning: bool
    versioning_use_md5: bool
    recompute_hashes: bool
    base_url: str | None
    script_name: str | None
    publisher_signature: str
    resources: Iterable[Dependable] | None

@type_check_only
class _InjectorPluginOptions(TypedDict, total=False):
    compile: bool
    bundle: bool
    rollup: bool
    debug: bool
    minified: bool

@type_check_only
class _TopBottomInjectorPluginOptions(_InjectorPluginOptions, total=False):
    bottom: bool
    force_bottom: bool

CONTENT_TYPES: list[str]

class Injector:
    app: WSGIApplication
    config: _NeededResourcesConfig
    injector: InjectorPlugin
    def __init__(
        self, app: WSGIApplication, injector: InjectorPlugin | None = None, **config: Unpack[_NeededResourcesConfig]
    ) -> None: ...
    def __call__(self, environ: WSGIEnvironment, start_response: StartResponse) -> Iterable[bytes]: ...

class InjectorPlugin:
    @property
    @abstractmethod
    def name(self) -> str: ...
    def __init__(self, options: _InjectorPluginOptions) -> None: ...
    def make_inclusion(self, needed: NeededResources, resources: set[Resource] | None = None) -> Inclusion: ...
    def __call__(
        self, html: bytes, needed: NeededResources, request: Request | None = None, response: Response | None = None
    ) -> None: ...

class TopBottomInjector(InjectorPlugin):
    name: Literal["topbottom"]
    def __init__(self, options: _TopBottomInjectorPluginOptions) -> None: ...
    def group(self, needed: NeededResources) -> tuple[Inclusion, Inclusion]: ...

def make_injector(app: WSGIApplication, global_config: Any, **local_config: Any) -> Injector: ...



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
Copyright 2016 The TensorFlow Authors. All Rights Reserved.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
==============================================================================
"""

import builtins
import typing

import google.protobuf.descriptor
import google.protobuf.message
import tensorflow.core.protobuf.cluster_pb2
import tensorflow.core.protobuf.config_pb2
import tensorflow.core.protobuf.device_filters_pb2

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ServerDef(google.protobuf.message.Message):
    """Defines the configuration of a single TensorFlow server."""

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CLUSTER_FIELD_NUMBER: builtins.int
    JOB_NAME_FIELD_NUMBER: builtins.int
    REPLICA_FIELD_NUMBER: builtins.int
    TASK_INDEX_FIELD_NUMBER: builtins.int
    DEFAULT_SESSION_CONFIG_FIELD_NUMBER: builtins.int
    PROTOCOL_FIELD_NUMBER: builtins.int
    PORT_FIELD_NUMBER: builtins.int
    CLUSTER_DEVICE_FILTERS_FIELD_NUMBER: builtins.int
    job_name: builtins.str
    """The name of the job of which this server is a member.

    NOTE(mrry): The `cluster` field must contain a `JobDef` with a `name` field
    that matches this name.
    """
    replica: builtins.int
    """Replica this server manages."""
    task_index: builtins.int
    """The task index of this server in its job.

    NOTE: The `cluster` field must contain a `JobDef` with a matching `name`
    and a mapping in its `tasks` field for this index.
    """
    protocol: builtins.str
    """The protocol to be used by this server.

    Acceptable values include: "grpc", "grpc+verbs".
    """
    port: builtins.int
    """The server port. If not set, then we identify the port from the job_name."""
    @property
    def cluster(self) -> tensorflow.core.protobuf.cluster_pb2.ClusterDef:
        """The cluster of which this server is a member."""

    @property
    def default_session_config(self) -> tensorflow.core.protobuf.config_pb2.ConfigProto:
        """The default configuration for sessions that run on this server."""

    @property
    def cluster_device_filters(self) -> tensorflow.core.protobuf.device_filters_pb2.ClusterDeviceFilters:
        """Device filters for remote tasks in the cluster.
        NOTE: This is an experimental feature and only effective in TensorFlow 2.x.
        """

    def __init__(
        self,
        *,
        cluster: tensorflow.core.protobuf.cluster_pb2.ClusterDef | None = ...,
        job_name: builtins.str | None = ...,
        replica: builtins.int | None = ...,
        task_index: builtins.int | None = ...,
        default_session_config: tensorflow.core.protobuf.config_pb2.ConfigProto | None = ...,
        protocol: builtins.str | None = ...,
        port: builtins.int | None = ...,
        cluster_device_filters: tensorflow.core.protobuf.device_filters_pb2.ClusterDeviceFilters | None = ...,
    ) -> None: ...
    def HasField(
        self,
        field_name: typing.Literal[
            "cluster",
            b"cluster",
            "cluster_device_filters",
            b"cluster_device_filters",
            "default_session_config",
            b"default_session_config",
        ],
    ) -> builtins.bool: ...
    def ClearField(
        self,
        field_name: typing.Literal[
            "cluster",
            b"cluster",
            "cluster_device_filters",
            b"cluster_device_filters",
            "default_session_config",
            b"default_session_config",
            "job_name",
            b"job_name",
            "port",
            b"port",
            "protocol",
            b"protocol",
            "replica",
            b"replica",
            "task_index",
            b"task_index",
        ],
    ) -> None: ...

global___ServerDef = ServerDef



from collections.abc import Callable
from typing import Final, TextIO
from typing_extensions import Self

CONTROL_SEQUENCES: Final[dict[str, str]]

def format_text(text: str, f: str) -> str: ...

class MultilinePrinterBase:
    def __init__(self, stream: TextIO | None = None, lines: int = 1) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *args: object) -> None: ...
    def print_at_line(self, text: str, pos: int) -> None: ...
    def end(self) -> None: ...
    def write(self, *text: str) -> None: ...

class QuietMultilinePrinter(MultilinePrinterBase): ...
class MultilineLogger(MultilinePrinterBase): ...
class BreaklineStatusPrinter(MultilinePrinterBase): ...

class MultilinePrinter(MultilinePrinterBase):
    def __init__(self, stream: TextIO | None = None, lines: int = 1, preserve_output: bool = True) -> None: ...
    def lock(func: Callable[..., object]) -> Callable[..., object]: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues]



from __future__ import annotations

import mmap
from typing import IO, AnyStr


def check_write(io_bytes: IO[bytes], io_str: IO[str], io_anystr: IO[AnyStr], any_str: AnyStr, buf: mmap.mmap) -> None:
    io_bytes.write(b"")
    io_bytes.write(buf)
    io_bytes.write("")  # type: ignore
    io_bytes.write(any_str)  # type: ignore

    io_str.write(b"")  # type: ignore
    io_str.write(buf)  # type: ignore
    io_str.write("")
    io_str.write(any_str)  # type: ignore

    io_anystr.write(b"")  # type: ignore
    io_anystr.write(buf)  # type: ignore
    io_anystr.write("")  # type: ignore
    io_anystr.write(any_str)



import abc
from dataclasses import dataclass
from functools import cached_property
from typing import Final

@dataclass(frozen=True)
class JsRuntimeInfo:
    name: str
    path: str
    version: str
    version_tuple: tuple[int, ...]
    supported: bool = True

class JsRuntime(abc.ABC):
    def __init__(self, path: str | None = None) -> None: ...
    @cached_property
    @abc.abstractmethod
    def info(self) -> JsRuntimeInfo | None: ...

class DenoJsRuntime(JsRuntime):
    MIN_SUPPORTED_VERSION: Final[tuple[int, int, int]]
    @cached_property
    def info(self) -> JsRuntimeInfo | None: ...

class BunJsRuntime(JsRuntime):
    MIN_SUPPORTED_VERSION: Final[tuple[int, int, int]]
    @cached_property
    def info(self) -> JsRuntimeInfo | None: ...

class NodeJsRuntime(JsRuntime):
    MIN_SUPPORTED_VERSION: Final[tuple[int, int, int]]
    @cached_property
    def info(self) -> JsRuntimeInfo | None: ...

class QuickJsRuntime(JsRuntime):
    MIN_SUPPORTED_VERSION: Final[tuple[int, int, int]]
    @cached_property
    def info(self) -> JsRuntimeInfo | None: ...



import itertools
import re
import zipfile
from _typeshed import FileDescriptorOrPath, Incomplete, ReadableBuffer, SizedBuffer, Unused
from collections.abc import Generator
from typing import ClassVar, Final, Literal
from xml.etree import ElementTree

from docutils import nodes, writers
from docutils.frontend import Values
from docutils.languages import _LanguageModule
from docutils.readers import standalone

__docformat__: Final = "reStructuredText"
VERSION: Final[str]
IMAGE_NAME_COUNTER: Final[itertools.count[int]]

class _ElementInterfaceWrapper(ElementTree.Element):
    def __init__(self, tag: str, attrib: dict[str, str] | None = None) -> None: ...
    def setparent(self, parent) -> None: ...
    def getparent(self): ...

SPACES_PATTERN: Final[re.Pattern[str]]
TABS_PATTERN: Final[re.Pattern[str]]
FILL_PAT1: Final[re.Pattern[str]]
FILL_PAT2: Final[re.Pattern[str]]
TABLESTYLEPREFIX: Final[str]
TABLENAMEDEFAULT: Final[str]
TABLEPROPERTYNAMES: Final[tuple[str, ...]]
GENERATOR_DESC: Final[str]
NAME_SPACE_1: Final[str]
CONTENT_NAMESPACE_DICT: Final[dict[str, str]]
CNSD: Final[dict[str, str]]
STYLES_NAMESPACE_DICT: Final[dict[str, str]]
SNSD: Final[dict[str, str]]
MANIFEST_NAMESPACE_DICT: Final[dict[str, str]]
MANNSD: Final[dict[str, str]]
META_NAMESPACE_DICT: Final[dict[str, str]]
METNSD: Final[dict[str, str]]
CONTENT_NAMESPACE_ATTRIB: Final[dict[str, str]]
STYLES_NAMESPACE_ATTRIB: Final[dict[str, str]]
MANIFEST_NAMESPACE_ATTRIB: Final[dict[str, str]]
META_NAMESPACE_ATTRIB: Final[dict[str, str]]

def Element(
    tag: str, attrib: dict[str, str] | None = None, nsmap: Unused = None, nsdict: Unused = ...
) -> _ElementInterfaceWrapper: ...
def SubElement(
    parent: _ElementInterfaceWrapper, tag: str, attrib: dict[str, str] | None = None, nsmap: Unused = None, nsdict: Unused = ...
) -> _ElementInterfaceWrapper: ...
def fix_ns(tag: str, attrib: dict[str, str], nsdict: Unused) -> tuple[str, dict[str, str]]: ...
def add_ns(tag: str, nsdict: Unused = ...) -> str: ...
def ToString(et: ElementTree.ElementTree) -> str: ...
def escape_cdata(text: str) -> str: ...

class TableStyle:
    border: str | None
    backgroundcolor: str | None
    def __init__(self, border: str | None = None, backgroundcolor: str | None = None) -> None: ...
    def get_border_(self): ...
    def set_border_(self, border) -> None: ...
    border_: Incomplete
    def get_backgroundcolor_(self): ...
    def set_backgroundcolor_(self, backgroundcolor) -> None: ...
    backgroundcolor_: Incomplete

BUILTIN_DEFAULT_TABLE_STYLE: TableStyle

class ListLevel:
    level: int
    sibling_level: bool
    nested_level: bool
    def __init__(self, level: int, sibling_level: bool = True, nested_level: bool = True) -> None: ...
    def get_sibling(self) -> bool: ...
    def set_sibling(self, sibling_level: bool) -> None: ...
    def get_nested(self) -> bool: ...
    def set_nested(self, nested_level: bool) -> None: ...
    def get_level(self) -> int: ...
    def set_level(self, level: int) -> None: ...

class Writer(writers.Writer[bytes]):
    MIME_TYPE: ClassVar[str]
    EXTENSION: ClassVar[str]
    default_stylesheet: ClassVar[str]
    default_stylesheet_path: ClassVar[str]
    default_template: ClassVar[str]
    default_template_path: ClassVar[str]
    translator_class: type[ODFTranslator]
    settings: Values
    visitor: ODFTranslator
    def assemble_my_parts(self) -> None: ...
    def update_stylesheet(self, stylesheet_root, language_code: str | None, region_code: str | None): ...
    def write_zip_str(
        self, zfile: zipfile.ZipFile, name: str, bytes_: str | SizedBuffer, compress_type: int = zipfile.ZIP_DEFLATED
    ) -> None: ...
    def store_embedded_files(self, zfile: zipfile.ZipFile) -> None: ...
    def get_settings(self) -> bytes: ...
    def get_stylesheet(self) -> bytes: ...
    def copy_from_stylesheet(self, outzipfile: zipfile.ZipFile) -> None: ...
    def assemble_parts(self) -> None: ...
    def create_manifest(self) -> str: ...
    def create_meta(self) -> str: ...

class ODFTranslator(nodes.GenericNodeVisitor):
    used_styles: ClassVar[tuple[str, ...]]
    settings: Values
    language_code: str
    language: _LanguageModule
    format_map: dict[str, str]
    section_level: int
    section_count: int
    content_tree: ElementTree.ElementTree
    current_element: _ElementInterfaceWrapper
    automatic_styles: _ElementInterfaceWrapper
    body_text_element: _ElementInterfaceWrapper
    paragraph_style_stack: list[str]
    list_style_stack: list[str]
    table_count: int
    column_count: int
    trace_level: int
    optiontablestyles_generated: bool
    field_name: Incomplete
    field_element: Incomplete
    title: str | None
    image_count: int
    image_style_count: int
    image_dict: dict[str, tuple[str, str]]
    embedded_file_list: list[Incomplete]
    syntaxhighlighting: int
    syntaxhighlight_lexer: str
    header_content: list[Incomplete]
    footer_content: list[Incomplete]
    in_header: bool
    in_footer: bool
    blockstyle: str
    in_table_of_contents: bool
    table_of_content_index_body: _ElementInterfaceWrapper | None
    list_level: int
    def_list_level: int
    footnote_ref_dict: dict[Incomplete, _ElementInterfaceWrapper]
    footnote_list: list[tuple[Incomplete, Incomplete]]
    footnote_chars_idx: int
    footnote_level: int
    pending_ids: Incomplete
    in_paragraph: bool
    found_doc_title: bool
    bumped_list_level_stack: list[ListLevel]
    meta_dict: dict[Incomplete, Incomplete]
    line_block_level: int
    line_indent_level: int
    citation_id: Incomplete
    style_index: int
    str_stylesheet: str | bytes
    str_stylesheetcontent: str | bytes | None
    dom_stylesheet: ElementTree.Element | None
    table_styles: dict[str, TableStyle] | None
    in_citation: bool
    inline_style_count_stack: list[int]
    def get_str_stylesheet(self) -> str | bytes: ...
    dom_stylesheetcontent: ElementTree.Element
    def retrieve_styles(self, extension: str) -> None: ...
    def extract_table_styles(self, styles_str: str | ReadableBuffer) -> dict[str, TableStyle]: ...
    def get_property(self, stylenode: ElementTree.Element) -> str | None: ...
    def add_doc_title(self) -> None: ...
    def find_first_text_p(self, el: _ElementInterfaceWrapper) -> _ElementInterfaceWrapper | None: ...
    def attach_page_style(self, el: _ElementInterfaceWrapper) -> None: ...
    def rststyle(self, name: str, parameters: tuple[object, ...] = ()) -> str: ...
    def generate_content_element(self, root: _ElementInterfaceWrapper) -> _ElementInterfaceWrapper: ...
    def setup_page(self): ...
    def get_dom_stylesheet(self) -> ElementTree.Element | None: ...
    def setup_paper(self, root_el) -> None: ...
    def add_header_footer(self, root_el) -> None: ...
    code_none: int
    code_field: int
    code_text: int
    field_pat: re.Pattern[str]
    def create_custom_headfoot(self, parent: _ElementInterfaceWrapper, text: str, style_name, automatic_styles) -> None: ...
    def make_field_element(
        self, parent: _ElementInterfaceWrapper, text: str, style_name, automatic_styles
    ) -> _ElementInterfaceWrapper | None: ...
    def split_field_specifiers_iter(self, text: str) -> Generator[tuple[int, str]]: ...
    def astext(self) -> str: ...
    def content_astext(self) -> str: ...
    def set_title(self, title: str) -> None: ...
    def get_title(self) -> str | None: ...
    def set_embedded_file_list(self, embedded_file_list) -> None: ...
    def get_embedded_file_list(self): ...
    def get_meta_dict(self): ...
    def process_footnotes(self) -> None: ...
    def append_child(
        self, tag: str, attrib: dict[str, str] | None = None, parent: _ElementInterfaceWrapper | None = None
    ) -> _ElementInterfaceWrapper: ...
    def append_p(self, style: str, text: str | None = None) -> _ElementInterfaceWrapper: ...
    def append_pending_ids(self, el: _ElementInterfaceWrapper) -> None: ...
    def set_current_element(self, el: _ElementInterfaceWrapper) -> None: ...
    def set_to_parent(self) -> None: ...
    def generate_labeled_block(self, node: nodes.Node, label: str) -> _ElementInterfaceWrapper: ...
    def generate_labeled_line(self, node: nodes.Node, label: str) -> _ElementInterfaceWrapper: ...
    def encode(self, text: str) -> str: ...
    def dispatch_visit(self, node: nodes.Element) -> None: ...  # type: ignore[override]
    def handle_basic_atts(self, node: nodes.Element) -> None: ...
    def default_visit(self, node: nodes.Element) -> None: ...  # type: ignore[override]
    def default_departure(self, node: nodes.Element) -> None: ...  # type: ignore[override]
    def visit_Text(self, node: nodes.Text) -> None: ...
    def depart_Text(self, node: nodes.Text) -> None: ...
    def visit_address(self, node: nodes.address) -> None: ...
    def depart_address(self, node: nodes.address) -> None: ...
    def visit_author(self, node: nodes.author) -> None: ...
    def depart_author(self, node: nodes.author) -> None: ...
    def visit_authors(self, node: nodes.authors) -> None: ...
    def depart_authors(self, node: nodes.authors) -> None: ...
    def visit_contact(self, node: nodes.contact) -> None: ...
    def depart_contact(self, node: nodes.contact) -> None: ...
    def visit_copyright(self, node: nodes.copyright) -> None: ...
    def depart_copyright(self, node: nodes.copyright) -> None: ...
    def visit_date(self, node: nodes.date) -> None: ...
    def depart_date(self, node: nodes.date) -> None: ...
    def visit_organization(self, node: nodes.organization) -> None: ...
    def depart_organization(self, node: nodes.organization) -> None: ...
    def visit_status(self, node: nodes.status) -> None: ...
    def depart_status(self, node: nodes.status) -> None: ...
    def visit_revision(self, node: nodes.revision) -> None: ...
    def depart_revision(self, node: nodes.revision) -> None: ...
    def visit_version(self, node: nodes.version) -> None: ...
    def depart_version(self, node: nodes.version) -> None: ...
    def visit_attribution(self, node: nodes.attribution) -> None: ...
    def depart_attribution(self, node: nodes.attribution) -> None: ...
    def visit_block_quote(self, node: nodes.block_quote) -> None: ...
    def depart_block_quote(self, node: nodes.block_quote) -> None: ...
    def visit_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def depart_bullet_list(self, node: nodes.bullet_list) -> None: ...
    def visit_caption(self, node: nodes.caption) -> None: ...
    def depart_caption(self, node: nodes.caption) -> None: ...
    def visit_comment(self, node: nodes.comment) -> None: ...
    def depart_comment(self, node: nodes.comment) -> None: ...
    def visit_compound(self, node: nodes.compound) -> None: ...
    def depart_compound(self, node: nodes.compound) -> None: ...
    def visit_container(self, node: nodes.container) -> None: ...
    def depart_container(self, node: nodes.container) -> None: ...
    def visit_decoration(self, node: nodes.decoration) -> None: ...
    def depart_decoration(self, node: nodes.decoration) -> None: ...
    def visit_definition_list(self, node: nodes.definition_list) -> None: ...
    def depart_definition_list(self, node: nodes.definition_list) -> None: ...
    def visit_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def depart_definition_list_item(self, node: nodes.definition_list_item) -> None: ...
    def visit_term(self, node: nodes.term) -> None: ...
    def depart_term(self, node: nodes.term) -> None: ...
    def visit_definition(self, node: nodes.definition) -> None: ...
    def depart_definition(self, node: nodes.definition) -> None: ...
    def visit_classifier(self, node: nodes.classifier) -> None: ...
    def depart_classifier(self, node: nodes.classifier) -> None: ...
    def visit_document(self, node: nodes.document) -> None: ...
    def depart_document(self, node: nodes.document) -> None: ...
    def visit_docinfo(self, node: nodes.docinfo) -> None: ...
    def depart_docinfo(self, node: nodes.docinfo) -> None: ...
    def visit_emphasis(self, node: nodes.emphasis) -> None: ...
    def depart_emphasis(self, node: nodes.emphasis) -> None: ...
    def visit_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def depart_enumerated_list(self, node: nodes.enumerated_list) -> None: ...
    def visit_list_item(self, node: nodes.list_item) -> None: ...
    def depart_list_item(self, node: nodes.list_item) -> None: ...
    def visit_header(self, node: nodes.header) -> None: ...
    def depart_header(self, node: nodes.header) -> None: ...
    def visit_footer(self, node: nodes.footer) -> None: ...
    def depart_footer(self, node: nodes.footer) -> None: ...
    def visit_field(self, node: nodes.field) -> None: ...
    def depart_field(self, node: nodes.field) -> None: ...
    def visit_field_list(self, node: nodes.field_list) -> None: ...
    def depart_field_list(self, node: nodes.field_list) -> None: ...
    def visit_field_name(self, node: nodes.field_name) -> None: ...
    def depart_field_name(self, node: nodes.field_name) -> None: ...
    def visit_field_body(self, node: nodes.field_body) -> None: ...
    def depart_field_body(self, node: nodes.field_body) -> None: ...
    def visit_figure(self, node: nodes.figure) -> None: ...
    def depart_figure(self, node: nodes.figure) -> None: ...
    save_footnote_current: _ElementInterfaceWrapper
    def visit_footnote(self, node: nodes.footnote) -> None: ...
    def depart_footnote(self, node: nodes.footnote) -> None: ...
    footnote_chars: list[str]
    def visit_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def depart_footnote_reference(self, node: nodes.footnote_reference) -> None: ...
    def visit_citation(self, node: nodes.citation) -> None: ...
    def depart_citation(self, node: nodes.citation) -> None: ...
    def visit_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def depart_citation_reference(self, node: nodes.citation_reference) -> None: ...
    def visit_label(self, node: nodes.label) -> None: ...
    def depart_label(self, node: nodes.label) -> None: ...
    def visit_generated(self, node: nodes.generated) -> None: ...
    def depart_generated(self, node: nodes.generated) -> None: ...
    def check_file_exists(self, path: FileDescriptorOrPath) -> Literal[1, 0]: ...
    def visit_image(self, node: nodes.image) -> None: ...
    def depart_image(self, node: nodes.image) -> None: ...
    def get_image_width_height(self, node: nodes.image, attr: str) -> tuple[float | None, Literal["%", "cm"] | None]: ...
    def convert_to_cm(self, size: str) -> tuple[float, Literal["cm"]]: ...
    def get_image_scale(self, node: nodes.image) -> float: ...
    def get_image_scaled_width_height(self, node: nodes.image, source) -> tuple[Incomplete, Incomplete]: ...
    def get_page_width(self) -> float: ...
    def generate_figure(self, node: nodes.image, source, destination, current_element): ...
    def generate_image(
        self, node: nodes.image, source, destination, current_element, frame_attrs: dict[str, str] | None = None
    ): ...
    def is_in_table(self, node: nodes.Node) -> bool: ...
    def visit_legend(self, node: nodes.legend) -> None: ...
    def depart_legend(self, node: nodes.legend) -> None: ...
    def visit_line_block(self, node: nodes.line_block) -> None: ...
    def depart_line_block(self, node: nodes.line_block) -> None: ...
    def visit_line(self, node: nodes.line) -> None: ...
    def depart_line(self, node: nodes.line) -> None: ...
    def visit_literal(self, node: nodes.literal) -> None: ...
    def depart_literal(self, node: nodes.literal) -> None: ...
    def visit_inline(self, node: nodes.inline) -> None: ...
    def depart_inline(self, node: nodes.inline) -> None: ...
    def fill_line(self, line: str) -> str: ...
    def fill_func1(self, matchobj: re.Match[str]) -> str: ...
    def fill_func2(self, matchobj: re.Match[str]) -> str: ...
    def visit_literal_block(self, node: nodes.literal_block) -> None: ...
    def depart_literal_block(self, node: nodes.literal_block) -> None: ...
    def visit_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def depart_doctest_block(self, node: nodes.doctest_block) -> None: ...
    def visit_math(self, node: nodes.math) -> None: ...
    def depart_math(self, node: nodes.math) -> None: ...
    def visit_math_block(self, node: nodes.math_block) -> None: ...
    def depart_math_block(self, node: nodes.math_block) -> None: ...
    def visit_meta(self, node: nodes.meta) -> None: ...
    def depart_meta(self, node: nodes.meta) -> None: ...
    def visit_option_list(self, node: nodes.option_list) -> None: ...
    def depart_option_list(self, node: nodes.option_list) -> None: ...
    def visit_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def depart_option_list_item(self, node: nodes.option_list_item) -> None: ...
    def visit_option_group(self, node: nodes.option_group) -> None: ...
    def depart_option_group(self, node: nodes.option_group) -> None: ...
    def visit_option(self, node: nodes.option) -> None: ...
    def depart_option(self, node: nodes.option) -> None: ...
    def visit_option_string(self, node: nodes.option_string) -> None: ...
    def depart_option_string(self, node: nodes.option_string) -> None: ...
    def visit_option_argument(self, node: nodes.option_argument) -> None: ...
    def depart_option_argument(self, node: nodes.option_argument) -> None: ...
    def visit_description(self, node: nodes.description) -> None: ...
    def depart_description(self, node: nodes.description) -> None: ...
    def visit_paragraph(self, node: nodes.paragraph) -> None: ...
    def depart_paragraph(self, node: nodes.paragraph) -> None: ...
    def visit_problematic(self, node: nodes.problematic) -> None: ...
    def depart_problematic(self, node: nodes.problematic) -> None: ...
    def visit_raw(self, node: nodes.raw) -> None: ...
    def depart_raw(self, node: nodes.raw) -> None: ...
    def visit_reference(self, node: nodes.reference) -> None: ...
    def depart_reference(self, node: nodes.reference) -> None: ...
    def visit_rubric(self, node: nodes.rubric) -> None: ...
    def depart_rubric(self, node: nodes.rubric) -> None: ...
    def visit_section(self, node: nodes.section, move_ids: int = 1) -> None: ...
    def depart_section(self, node: nodes.section) -> None: ...
    def visit_strong(self, node: nodes.strong) -> None: ...
    def depart_strong(self, node: nodes.strong) -> None: ...
    def visit_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def depart_substitution_definition(self, node: nodes.substitution_definition) -> None: ...
    def visit_system_message(self, node: nodes.system_message) -> None: ...
    def depart_system_message(self, node: nodes.system_message) -> None: ...
    def get_table_style(self, node) -> TableStyle: ...
    current_table_style: _ElementInterfaceWrapper
    table_width: float
    def visit_table(self, node: nodes.table) -> None: ...
    def depart_table(self, node: nodes.table) -> None: ...
    def visit_tgroup(self, node: nodes.tgroup) -> None: ...
    def depart_tgroup(self, node: nodes.tgroup) -> None: ...
    def visit_colspec(self, node: nodes.colspec) -> None: ...
    def depart_colspec(self, node: nodes.colspec) -> None: ...
    in_thead: bool
    def visit_thead(self, node: nodes.thead) -> None: ...
    def depart_thead(self, node: nodes.thead) -> None: ...
    def visit_row(self, node: nodes.row) -> None: ...
    def depart_row(self, node: nodes.row) -> None: ...
    def visit_entry(self, node: nodes.entry) -> None: ...
    def depart_entry(self, node: nodes.entry) -> None: ...
    def visit_tbody(self, node: nodes.tbody) -> None: ...
    def depart_tbody(self, node: nodes.tbody) -> None: ...
    def visit_target(self, node: nodes.target) -> None: ...
    def depart_target(self, node: nodes.target) -> None: ...
    def visit_title(self, node: nodes.title, move_ids: int = 1, title_type: str = "title") -> None: ...
    def depart_title(self, node: nodes.title) -> None: ...
    def visit_subtitle(self, node: nodes.subtitle, move_ids: int = 1) -> None: ...
    def depart_subtitle(self, node: nodes.subtitle) -> None: ...
    def visit_title_reference(self, node: nodes.title_reference) -> None: ...
    def depart_title_reference(self, node: nodes.title_reference) -> None: ...
    def generate_table_of_content_entry_template(self, el1) -> None: ...
    def find_title_label(self, node, class_type, label_key): ...
    save_current_element: Incomplete
    def visit_topic(self, node: nodes.topic) -> None: ...
    def depart_topic(self, node: nodes.topic) -> None: ...
    def update_toc_page_numbers(self, el) -> None: ...
    def update_toc_collect(self, el, level, collection) -> None: ...
    def update_toc_add_numbers(self, collection) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...
    def depart_transition(self, node: nodes.transition) -> None: ...
    def visit_warning(self, node: nodes.warning) -> None: ...
    def depart_warning(self, node: nodes.warning) -> None: ...
    def visit_attention(self, node: nodes.attention) -> None: ...
    def depart_attention(self, node: nodes.attention) -> None: ...
    def visit_caution(self, node: nodes.caution) -> None: ...
    def depart_caution(self, node: nodes.caution) -> None: ...
    def visit_danger(self, node: nodes.danger) -> None: ...
    def depart_danger(self, node: nodes.danger) -> None: ...
    def visit_error(self, node: nodes.error) -> None: ...
    def depart_error(self, node: nodes.error) -> None: ...
    def visit_hint(self, node: nodes.hint) -> None: ...
    def depart_hint(self, node: nodes.hint) -> None: ...
    def visit_important(self, node: nodes.important) -> None: ...
    def depart_important(self, node: nodes.important) -> None: ...
    def visit_note(self, node: nodes.note) -> None: ...
    def depart_note(self, node: nodes.note) -> None: ...
    def visit_tip(self, node: nodes.tip) -> None: ...
    def depart_tip(self, node: nodes.tip) -> None: ...
    def visit_admonition(self, node: nodes.admonition) -> None: ...
    def depart_admonition(self, node: nodes.admonition) -> None: ...
    def generate_admonition(self, node: nodes.Node, label: str, title: str | None = None) -> None: ...
    def visit_subscript(self, node: nodes.subscript) -> None: ...
    def depart_subscript(self, node: nodes.subscript) -> None: ...
    def visit_superscript(self, node: nodes.superscript) -> None: ...
    def depart_superscript(self, node: nodes.superscript) -> None: ...
    def visit_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def depart_abbreviation(self, node: nodes.abbreviation) -> None: ...
    def visit_acronym(self, node: nodes.acronym) -> None: ...
    def depart_acronym(self, node: nodes.acronym) -> None: ...
    def visit_sidebar(self, node: nodes.sidebar) -> None: ...
    def depart_sidebar(self, node: nodes.sidebar) -> None: ...

class Reader(standalone.Reader[str | bytes]): ...



from _typeshed import OpenBinaryModeUpdating, OpenTextModeReading, OpenTextModeWriting, SupportsWrite
from collections.abc import Callable
from typing import IO, Any, Protocol, TypeVar, type_check_only

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from pygraphviz.agraph import AGraph  # type: ignore[import-not-found]  # pyright: ignore[reportMissingImports]

__all__ = ["from_agraph", "to_agraph", "write_dot", "read_dot", "graphviz_layout", "pygraphviz_layout", "view_pygraphviz"]

_ModeT_contra = TypeVar("_ModeT_contra", bound=str, contravariant=True)
_FileT_co = TypeVar("_FileT_co", covariant=True)

@type_check_only
class _SupportsOpen(Protocol[_ModeT_contra, _FileT_co]):
    def open(self, *, mode: _ModeT_contra) -> _FileT_co: ...

@_dispatchable
def from_agraph(
    A: AGraph, create_using: Graph[str] | type[Graph[str]] | None = None  # TODO: add overloads on `create_using`
) -> Graph[str]: ...
def to_agraph(N: Graph[_Node]) -> AGraph: ...
def write_dot(
    G: Graph[_Node], path: str | IO[str] | IO[bytes] | _SupportsOpen[OpenTextModeWriting, IO[str] | IO[bytes]]
) -> None: ...
@_dispatchable
def read_dot(path: str | IO[str] | IO[bytes] | _SupportsOpen[OpenTextModeReading, IO[str] | IO[bytes]]) -> Graph[str]: ...
def graphviz_layout(
    G: Graph[_Node], prog: str = "neato", root: str | None = None, args: str = ""
) -> dict[_Node, tuple[float, float]]: ...
def pygraphviz_layout(
    G: Graph[_Node], prog: str = "neato", root: str | None = None, args: str = ""
) -> dict[_Node, tuple[float, float]]: ...
def view_pygraphviz(
    G: Graph[_Node],
    # From implementation looks like Callable could return object since it's always immediately stringified
    # But judging by documentation this seems like an extra runtime safety thing and not intended
    # Leaving as str unless anyone reports a valid use-case
    edgelabel: str | Callable[[dict[str, Any]], str] | None = None,
    prog: str = "dot",
    args: str = "",
    suffix: str = "",
    path: str | SupportsWrite[bytes] | _SupportsOpen[OpenBinaryModeUpdating, SupportsWrite[bytes]] | None = None,
    show: bool = True,
) -> tuple[str, AGraph]: ...



from grpc_reflection.v1alpha import reflection_pb2_grpc

class BaseReflectionServicer(reflection_pb2_grpc.ServerReflectionServicer):
    def __init__(self, service_names, pool=None) -> None: ...

__all__ = ["BaseReflectionServicer"]



from _typeshed import Unused
from typing import Literal, TypedDict, type_check_only
from typing_extensions import TypeAlias

@type_check_only
class _ContinueRequestResult(TypedDict):
    allThreadsContinued: bool

_Granularity: TypeAlias = Literal["statement", "instruction"]

def next(*, threadId: int, singleThread: bool = False, granularity: _Granularity = "statement", **args: Unused) -> None: ...
def step_in(*, threadId: int, singleThread: bool = False, granularity: _Granularity = "statement", **args: Unused) -> None: ...
def step_out(*, threadId: int, singleThread: bool = False, **args: Unused): ...
def continue_request(*, threadId: int, singleThread: bool = False, **args: Unused) -> _ContinueRequestResult: ...



from typing import Final

from authlib.jose.rfc7516 import JWEZipAlgorithm

GZIP_HEAD: Final[bytes]
MAX_SIZE: Final = 256000

class DeflateZipAlgorithm(JWEZipAlgorithm):
    name: str
    description: str
    def compress(self, s: bytes) -> bytes: ...
    def decompress(self, s: bytes) -> bytes: ...

def register_jwe_rfc7518() -> None: ...



from _typeshed import Incomplete
from typing_extensions import Self

class PayPalPayeeInput:
    def __init__(self, email_address: str | None = None, client_id: str | None = None) -> None: ...
    def to_graphql_variables(self) -> dict[str, Incomplete]: ...
    @staticmethod
    def builder() -> Builder: ...

    class Builder:
        def __init__(self) -> None: ...
        def email_address(self, email_address: str) -> Self: ...
        def client_id(self, client_id: str) -> Self: ...
        def build(self) -> PayPalPayeeInput: ...



from _typeshed import Incomplete
from abc import ABCMeta
from collections.abc import Iterable
from typing import ClassVar

class JWEAlgorithmBase(metaclass=ABCMeta):
    EXTRA_HEADERS: ClassVar[Iterable[str] | None]
    name: str | None
    description: str | None
    algorithm_type: str
    algorithm_location: str
    def prepare_key(self, raw_data) -> None: ...
    def generate_preset(self, enc_alg, key) -> None: ...

class JWEAlgorithm(JWEAlgorithmBase, metaclass=ABCMeta):
    def wrap(self, enc_alg, headers, key, preset=None) -> None: ...
    def unwrap(self, enc_alg, ek, headers, key) -> None: ...

class JWEAlgorithmWithTagAwareKeyAgreement(JWEAlgorithmBase, metaclass=ABCMeta):
    def generate_keys_and_prepare_headers(self, enc_alg, key, sender_key, preset=None) -> None: ...
    def agree_upon_key_and_wrap_cek(self, enc_alg, headers, key, sender_key, epk, cek, tag) -> None: ...
    def wrap(self, enc_alg, headers, key, sender_key, preset=None) -> None: ...
    def unwrap(self, enc_alg, ek, headers, key, sender_key, tag=None) -> None: ...

class JWEEncAlgorithm:
    name: str | None
    description: str | None
    algorithm_type: str
    algorithm_location: str
    IV_SIZE: Incomplete
    CEK_SIZE: Incomplete
    def generate_cek(self): ...
    def generate_iv(self): ...
    def check_iv(self, iv) -> None: ...
    def encrypt(self, msg, aad, iv, key) -> None: ...
    def decrypt(self, ciphertext, aad, iv, tag, key) -> None: ...

class JWEZipAlgorithm:
    name: Incomplete
    description: Incomplete
    algorithm_type: str
    algorithm_location: str
    def compress(self, s: bytes) -> bytes | None: ...
    def decompress(self, s: bytes) -> bytes | None: ...

class JWESharedHeader(dict[str, object]):
    protected: Incomplete
    unprotected: Incomplete
    def __init__(self, protected, unprotected) -> None: ...
    def update_protected(self, addition) -> None: ...
    @classmethod
    def from_dict(cls, obj): ...

class JWEHeader(dict[str, object]):
    protected: Incomplete
    unprotected: Incomplete
    header: Incomplete
    def __init__(self, protected, unprotected, header) -> None: ...



import types
from collections.abc import Callable, Collection
from typing import Any, NamedTuple

class _Package(NamedTuple):
    name: str
    version: str

def get_package_info(module: types.ModuleType) -> _Package: ...

class EnhancedModule(types.ModuleType):
    def __bool__(self) -> bool: ...
    def __getattribute__(self, attr: str) -> Any: ...

def passthrough_module(
    parent: types.ModuleType,
    child: str | types.ModuleType,
    allowed_attributes: Collection[str] = ...,
    *,
    callback: Callable[[str], object] = ...,
) -> types.ModuleType: ...



import datetime
from abc import abstractmethod
from typing import Any, overload

class BaseTzInfo(datetime.tzinfo):
    _utcoffset: datetime.timedelta | None
    _tzname: str | None
    zone: str | None  # Actually None but should be set on concrete subclasses
    # The following abstract methods don't exist in the implementation, but
    # are implemented by all sub-classes.
    @abstractmethod
    def localize(self, dt: datetime.datetime, is_dst: bool | None = ...) -> datetime.datetime: ...
    @abstractmethod
    def normalize(self, dt: datetime.datetime) -> datetime.datetime: ...
    @abstractmethod
    def tzname(self, dt: datetime.datetime | None, /) -> str: ...
    @abstractmethod
    def utcoffset(self, dt: datetime.datetime | None, /) -> datetime.timedelta | None: ...
    @abstractmethod
    def dst(self, dt: datetime.datetime | None, /) -> datetime.timedelta | None: ...

class StaticTzInfo(BaseTzInfo):
    def fromutc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def localize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def tzname(self, dt: datetime.datetime | None, is_dst: bool | None = None) -> str: ...
    def utcoffset(self, dt: datetime.datetime | None, is_dst: bool | None = None) -> datetime.timedelta: ...
    def dst(self, dt: datetime.datetime | None, is_dst: bool | None = None) -> datetime.timedelta: ...

class DstTzInfo(BaseTzInfo):
    def __init__(self, _inf: Any = None, _tzinfos: Any = None) -> None: ...
    def fromutc(self, dt: datetime.datetime) -> datetime.datetime: ...
    def localize(self, dt: datetime.datetime, is_dst: bool | None = False) -> datetime.datetime: ...
    def normalize(self, dt: datetime.datetime) -> datetime.datetime: ...
    def tzname(self, dt: datetime.datetime | None, is_dst: bool | None = None) -> str: ...
    # https://github.com/python/mypy/issues/12379
    @overload  # type: ignore[override]
    def utcoffset(self, dt: None, is_dst: bool | None = None) -> None: ...
    @overload
    def utcoffset(self, dt: datetime.datetime, is_dst: bool | None = None) -> datetime.timedelta: ...
    def dst(self, dt: datetime.datetime | None, is_dst: bool | None = None) -> datetime.timedelta | None: ...

__all__: list[str] = []



from typing import Any, TypedDict, type_check_only

import click

@type_check_only
class _FormData(TypedDict):
    command: click.Command
    fields: list[dict[str, Any]]  # each item is result of resources.input_fields.get_input_field() function

def get_form_for(command_path: str) -> str: ...
def _get_commands_by_path(command_path: str) -> list[tuple[click.Context, click.Command]]: ...
def _generate_form_data(ctx_and_commands: list[tuple[click.Context, click.Command]]) -> list[_FormData]: ...
def _process_help(help_text: bool) -> str: ...



from collections.abc import Awaitable
from typing import Any, ClassVar, Protocol, TypedDict, type_check_only

from asgiref.typing import ASGIReceiveCallable, ASGISendCallable, Scope, WebSocketScope
from channels.auth import UserLazyObject
from channels.db import database_sync_to_async
from channels.layers import BaseChannelLayer
from django.contrib.sessions.backends.base import SessionBase
from django.utils.functional import LazyObject

# _LazySession is a LazyObject that wraps a SessionBase instance.
# We subclass both for type checking purposes to expose SessionBase attributes,
# and suppress mypy's "misc" error with `# type: ignore[misc]`.
@type_check_only
class _LazySession(SessionBase, LazyObject):  # type: ignore[misc]
    _wrapped: SessionBase

@type_check_only
class _URLRoute(TypedDict):
    # Values extracted from Django's URLPattern matching,
    # passed through ASGI scope routing.
    # `args` and `kwargs` are the result of pattern matching against the URL path.
    args: tuple[Any, ...]
    kwargs: dict[str, Any]

# Channel Scope definition
@type_check_only
class _ChannelScope(WebSocketScope, total=False):
    # Channels specific
    channel: str
    url_route: _URLRoute
    path_remaining: str

    # Auth specific
    cookies: dict[str, str]
    session: _LazySession
    user: UserLazyObject | None

# Accepts any ASGI message dict with a required "type" key (str),
# but allows additional arbitrary keys for flexibility.
def get_handler_name(message: dict[str, Any]) -> str: ...
@type_check_only
class _ASGIApplicationProtocol(Protocol):
    consumer_class: AsyncConsumer

    # Accepts any initialization kwargs passed to the consumer class.
    # Typed as `Any` to allow flexibility in subclass-specific arguments.
    consumer_initkwargs: Any

    def __call__(self, scope: Scope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> Awaitable[None]: ...

class AsyncConsumer:
    channel_layer_alias: ClassVar[str]

    scope: _ChannelScope
    channel_layer: BaseChannelLayer
    channel_name: str
    channel_receive: ASGIReceiveCallable
    base_send: ASGISendCallable

    async def __call__(self, scope: _ChannelScope, receive: ASGIReceiveCallable, send: ASGISendCallable) -> None: ...
    async def dispatch(self, message: dict[str, Any]) -> None: ...
    async def send(self, message: dict[str, Any]) -> None: ...

    # initkwargs will be used to instantiate the consumer instance.
    @classmethod
    def as_asgi(cls, **initkwargs: Any) -> _ASGIApplicationProtocol: ...

class SyncConsumer(AsyncConsumer):

    # Since we're overriding asynchronous methods with synchronous ones,
    # we need to use `# type: ignore[override]` to suppress mypy errors.
    @database_sync_to_async
    def dispatch(self, message: dict[str, Any]) -> None: ...  # type: ignore[override]
    def send(self, message: dict[str, Any]) -> None: ...  # type: ignore[override]



from collections.abc import Collection

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable
from scipy.sparse import csr_array  # type: ignore[import-untyped]  # pyright: ignore[reportMissingImports]

__all__ = ["bethe_hessian_matrix"]

@_dispatchable
def bethe_hessian_matrix(G: Graph[_Node], r: float | None = None, nodelist: Collection[_Node] | None = None) -> csr_array: ...



from io import BytesIO

from waitress.buffers import OverflowableBuffer
from waitress.utilities import BadRequest

class FixedStreamReceiver:
    completed: bool
    error: None
    remain: int
    buf: OverflowableBuffer
    def __init__(self, cl: int, buf: OverflowableBuffer) -> None: ...
    def __len__(self) -> int: ...
    def received(self, data: bytes) -> int: ...
    def getfile(self) -> BytesIO: ...
    def getbuf(self) -> OverflowableBuffer: ...

class ChunkedReceiver:
    chunk_remainder: int
    validate_chunk_end: bool
    control_line: bytes
    chunk_end: bytes
    all_chunks_received: bool
    trailer: bytes
    completed: bool
    error: BadRequest | None
    buf: OverflowableBuffer
    def __init__(self, buf: OverflowableBuffer) -> None: ...
    def __len__(self) -> int: ...
    def received(self, s: bytes) -> int: ...
    def getfile(self) -> BytesIO: ...
    def getbuf(self) -> OverflowableBuffer: ...



from typing import Any

ECO_VERSION: str
HAVE_URANDOM: bool
INSTANCE_ID: str
IS_64BIT: bool
HAVE_UCS4: bool
HAVE_READLINE: bool
SQLITE_VERSION: str
OPENSSL_VERSION: str
TKINTER_VERSION: str
ZLIB_VERSION: str
EXPAT_VERSION: str
CPU_COUNT: int
HAVE_THREADING: bool
HAVE_IPV6: bool
RLIMIT_FDS_SOFT: int
RLIMIT_FDS_HARD: int
START_TIME_INFO: dict[str, str | float]

def getrandbits(k: int) -> int: ...
def get_python_info() -> dict[str, Any]: ...
def get_profile(**kwargs) -> dict[str, Any]: ...
def get_profile_json(indent: bool = False) -> str: ...
def main() -> None: ...
def dumps(val: Any, indent: int) -> str: ...



import threading
from _typeshed import Unused
from collections.abc import Callable
from typing import Final

import isapi.simple

ISAPI_REQUEST: Final = 1
ISAPI_SHUTDOWN: Final = 2

class WorkerThread(threading.Thread):
    running: bool
    io_req_port: int
    extension: ThreadPoolExtension
    def __init__(self, extension: ThreadPoolExtension, io_req_port: int) -> None: ...
    def call_handler(self, cblock) -> None: ...

class ThreadPoolExtension(isapi.simple.SimpleExtension):
    max_workers: int
    worker_shutdown_wait: int
    workers: list[WorkerThread]
    dispatch_map: dict[int, Callable[..., Unused]]
    io_req_port: int
    def GetExtensionVersion(self, vi) -> None: ...
    def HttpExtensionProc(self, control_block) -> int: ...
    def TerminateExtension(self, status) -> None: ...
    def DispatchConnection(self, errCode, bytes, key, overlapped) -> None: ...
    def Dispatch(self, ecb) -> None: ...
    def HandleDispatchError(self, ecb) -> None: ...



from collections.abc import Callable, Sequence
from datetime import date, datetime, time
from typing import Any
from typing_extensions import Self

from wtforms.fields.core import Field, _Filter, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext

__all__ = ("DateTimeField", "DateField", "TimeField", "MonthField", "DateTimeLocalField", "WeekField")

class DateTimeField(Field):
    format: list[str]
    strptime_format: list[str]
    data: datetime | None
    default: datetime | Callable[[], datetime] | None
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = "%Y-%m-%d %H:%M:%S",
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: datetime | Callable[[], datetime] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class DateField(DateTimeField):
    data: date | None  # type: ignore[assignment]
    default: date | Callable[[], date] | None  # type: ignore[assignment]
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = "%Y-%m-%d",
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: date | Callable[[], date] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class TimeField(DateTimeField):
    data: time | None  # type: ignore[assignment]
    default: time | Callable[[], time] | None  # type: ignore[assignment]
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = "%H:%M",
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: time | Callable[[], time] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class MonthField(DateField):
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = "%Y-%m",
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: time | Callable[[], time] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class WeekField(DateField):
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = "%Y-W%W",  # only difference is the default value
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: time | Callable[[], time] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class DateTimeLocalField(DateTimeField):
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        format: str | list[str] = ...,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: time | Callable[[], time] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...



import logging

from hvac.api.vault_api_base import VaultApiBase

DEFAULT_MOUNT_POINT: str

logger: logging.Logger

class Azure(VaultApiBase):
    def configure(self, tenant_id, resource, environment=None, client_id=None, client_secret=None, mount_point="azure"): ...
    def read_config(self, mount_point="azure"): ...
    def delete_config(self, mount_point="azure"): ...
    def create_role(
        self,
        name,
        policies=None,
        ttl=None,
        max_ttl=None,
        period=None,
        bound_service_principal_ids=None,
        bound_group_ids=None,
        bound_locations=None,
        bound_subscription_ids=None,
        bound_resource_groups=None,
        bound_scale_sets=None,
        num_uses=None,
        mount_point="azure",
    ): ...
    def read_role(self, name, mount_point="azure"): ...
    def list_roles(self, mount_point="azure"): ...
    def delete_role(self, name, mount_point="azure"): ...
    def login(
        self,
        role,
        jwt,
        subscription_id=None,
        resource_group_name=None,
        vm_name=None,
        vmss_name=None,
        use_token: bool = True,
        mount_point="azure",
    ): ...



import datetime

from ..core import WesternCalendar

class Barbados(WesternCalendar):
    non_computable_holiday_dict: dict[int, list[tuple[datetime.date, str]]]
    def get_kadooment_day(self, year: int) -> tuple[datetime.date, str]: ...
    def get_emancipation_day(self, year: int) -> tuple[datetime.date, str]: ...
    def non_computable_holiday(self, year: int) -> list[tuple[datetime.date, str]] | None: ...
    def get_fixed_holidays(self, year: int) -> list[tuple[datetime.date, str]]: ...



from __future__ import annotations

import sys
import types
from collections import UserDict
from typing import Any, Literal, TypeVar, Union
from typing_extensions import assert_type

_T = TypeVar("_T")

# test `types.SimpleNamespace`

# Valid:
types.SimpleNamespace()
types.SimpleNamespace(x=1, y=2)

if sys.version_info >= (3, 13):
    types.SimpleNamespace(())
    types.SimpleNamespace([])
    types.SimpleNamespace([("x", "y"), ("z", 1)])
    types.SimpleNamespace({})
    types.SimpleNamespace(UserDict({"x": 1, "y": 2}))


# Invalid:
types.SimpleNamespace(1)  # type: ignore
types.SimpleNamespace([1])  # type: ignore
types.SimpleNamespace([["x"]])  # type: ignore
types.SimpleNamespace(**{1: 2})  # type: ignore
types.SimpleNamespace({1: 2})  # type: ignore
types.SimpleNamespace([[1, 2]])  # type: ignore
types.SimpleNamespace(UserDict({1: 2}))  # type: ignore
types.SimpleNamespace([[[], 2]])  # type: ignore

# test: `types.MappingProxyType`
mp = types.MappingProxyType({1: 2, 3: 4})
mp.get("x")  # type: ignore
item = mp.get(1)
assert_type(item, Union[int, None])
item_2 = mp.get(2, 0)
assert_type(item_2, int)
item_3 = mp.get(3, "default")
assert_type(item_3, Union[int, str])
# Default isn't accepted as a keyword argument.
mp.get(4, default="default")  # type: ignore


# test: `types.DynamicClassAttribute`
class DCAtest:
    _value: int | None = None

    @types.DynamicClassAttribute
    def foo(self) -> int | None:
        return self._value

    @foo.setter
    def foo(self, value: int) -> None:
        self._value = value

    @foo.deleter
    def foo(self) -> None:
        self._value = None


# check that NotImplemented is treated as an "Any"
x: int = NotImplemented

if sys.version_info >= (3, 10):
    # test NotImplementedType usage
    assert_type(NotImplemented, types.NotImplementedType)
    assert_type(types.NotImplementedType(), types.NotImplementedType)
    # test EllipsisType usage
    assert_type(Ellipsis, types.EllipsisType)
    assert_type(types.EllipsisType(), types.EllipsisType)
    # test NoneType usage (disabled, passes with pyright, but mypy errors
    # assert_type(None, types.NoneType)
    # assert_type(types.NoneType(), types.NoneType)

if sys.version_info >= (3, 11):
    union_type = int | list[_T]

    # ideally this would be `_SpecialForm` (Union)
    assert_type(union_type | Literal[1], types.UnionType | Any)
    # Both mypy and pyright special-case this operation,
    # but in different ways, so we just check that no error is emitted:
    _ = union_type[int]



from _typeshed import Incomplete, SupportsGetItem

from networkx.classes.digraph import DiGraph
from networkx.classes.graph import _Node
from networkx.utils.backends import _dispatchable

__all__ = ["global_reaching_centrality", "local_reaching_centrality"]

@_dispatchable
def global_reaching_centrality(G: DiGraph[_Node], weight: str | None = None, normalized: bool | None = True) -> float: ...
@_dispatchable
def local_reaching_centrality(
    G: DiGraph[_Node],
    v: _Node,
    paths: SupportsGetItem[Incomplete, Incomplete] | None = None,
    weight: str | None = None,
    normalized: bool | None = True,
) -> float: ...



import sys
from _typeshed import ReadableBuffer
from collections.abc import Mapping
from logging import _ExcInfoType
from socket import _RetAddress, socket
from threading import Thread
from typing import Final, Protocol, type_check_only

from paramiko.channel import Channel
from paramiko.message import Message, _LikeBytes
from paramiko.pkey import PKey
from paramiko.transport import Transport

@type_check_only
class _AgentProxy(Protocol):
    def connect(self) -> None: ...
    def close(self) -> None: ...

cSSH2_AGENTC_REQUEST_IDENTITIES: Final[bytes]
SSH2_AGENT_IDENTITIES_ANSWER: Final = 12
cSSH2_AGENTC_SIGN_REQUEST: Final[bytes]
SSH2_AGENT_SIGN_RESPONSE: Final = 14

SSH_AGENT_RSA_SHA2_256: Final = 2
SSH_AGENT_RSA_SHA2_512: Final = 4
ALGORITHM_FLAG_MAP: Final[dict[str, int]]
key: str
value: int

class AgentSSH:
    def __init__(self) -> None: ...
    def get_keys(self) -> tuple[AgentKey, ...]: ...

class AgentProxyThread(Thread):
    def __init__(self, agent: _AgentProxy) -> None: ...
    def run(self) -> None: ...

class AgentLocalProxy(AgentProxyThread):
    def __init__(self, agent: AgentServerProxy) -> None: ...
    def get_connection(self) -> tuple[socket, _RetAddress]: ...

class AgentRemoteProxy(AgentProxyThread):
    def __init__(self, agent: AgentClientProxy, chan: Channel) -> None: ...
    def get_connection(self) -> tuple[socket, _RetAddress]: ...

if sys.platform == "win32":
    from .win_openssh import OpenSSHAgentConnection
    from .win_pageant import PageantConnection

    def get_agent_connection() -> PageantConnection | OpenSSHAgentConnection | None: ...

else:
    def get_agent_connection() -> socket | None: ...

class AgentClientProxy:
    thread: Thread
    def __init__(self, chanRemote: Channel) -> None: ...
    def __del__(self) -> None: ...
    def connect(self) -> None: ...
    def close(self) -> None: ...

class AgentServerProxy(AgentSSH):
    thread: Thread
    def __init__(self, t: Transport) -> None: ...
    def __del__(self) -> None: ...
    def connect(self) -> None: ...
    def close(self) -> None: ...
    def get_env(self) -> dict[str, str]: ...

class AgentRequestHandler:
    def __init__(self, chanClient: Channel) -> None: ...
    def __del__(self) -> None: ...
    def close(self) -> None: ...

class Agent(AgentSSH):
    def __init__(self) -> None: ...
    def close(self) -> None: ...

class AgentKey(PKey):
    agent: AgentSSH
    blob: bytes
    public_blob: None
    name: str
    comment: str
    def __init__(self, agent: AgentSSH, blob: ReadableBuffer, comment: str = "") -> None: ...
    def log(
        self,
        level: int,
        msg: object,
        *args: object,
        exc_info: _ExcInfoType = None,
        stack_info: bool = False,
        stacklevel: int = 1,
        extra: Mapping[str, object] | None = None,
    ) -> None: ...
    def asbytes(self) -> bytes: ...
    def get_name(self) -> str: ...
    def sign_ssh_data(self, data: _LikeBytes, algorithm: str | None = None) -> Message: ...



from _typeshed import Incomplete

from pony.orm.core import Database, Entity

db: Database

class Faculty(Entity):
    __slots__ = ()
    number: Incomplete
    name: Incomplete
    departments: Incomplete

class Department(Entity):
    __slots__ = ()
    number: Incomplete
    name: Incomplete
    faculty: Incomplete
    teachers: Incomplete
    majors: Incomplete
    groups: Incomplete

class Group(Entity):
    __slots__ = ()
    number: Incomplete
    grad_year: Incomplete
    department: Incomplete
    lessons: Incomplete
    students: Incomplete

class Student(Entity):
    __slots__ = ()
    name: Incomplete
    scholarship: Incomplete
    group: Incomplete
    grades: Incomplete

class Major(Entity):
    __slots__ = ()
    name: Incomplete
    department: Incomplete
    courses: Incomplete

class Subject(Entity):
    __slots__ = ()
    name: Incomplete
    courses: Incomplete
    teachers: Incomplete

class Course(Entity):
    __slots__ = ()
    major: Incomplete
    subject: Incomplete
    semester: Incomplete
    lect_hours: Incomplete
    pract_hours: Incomplete
    credit: Incomplete
    lessons: Incomplete
    grades: Incomplete

class Lesson(Entity):
    __slots__ = ()
    day_of_week: Incomplete
    meeting_time: Incomplete
    classroom: Incomplete
    course: Incomplete
    teacher: Incomplete
    groups: Incomplete

class Grade(Entity):
    __slots__ = ()
    student: Incomplete
    course: Incomplete
    teacher: Incomplete
    date: Incomplete
    value: Incomplete

class Teacher(Entity):
    __slots__ = ()
    name: Incomplete
    degree: Incomplete
    department: Incomplete
    subjects: Incomplete
    lessons: Incomplete
    grades: Incomplete

class Building(Entity):
    __slots__ = ()
    number: Incomplete
    description: Incomplete
    classrooms: Incomplete

class Classroom(Entity):
    __slots__ = ()
    building: Incomplete
    number: Incomplete
    description: Incomplete
    lessons: Incomplete

def test_queries() -> None: ...



from _typeshed import Incomplete
from types import FunctionType

from networkx.utils.backends import _dispatchable

__all__ = [
    "categorical_node_match",
    "categorical_edge_match",
    "categorical_multiedge_match",
    "numerical_node_match",
    "numerical_edge_match",
    "numerical_multiedge_match",
    "generic_node_match",
    "generic_edge_match",
    "generic_multiedge_match",
]

def copyfunc(f, name=None) -> FunctionType: ...
def allclose(x, y, rtol: float = 1.0000000000000001e-05, atol=1e-08) -> bool: ...
@_dispatchable
def categorical_node_match(attr, default): ...

categorical_edge_match: Incomplete

@_dispatchable
def categorical_multiedge_match(attr, default): ...
@_dispatchable
def numerical_node_match(attr, default, rtol: float = 1e-05, atol: float = 1e-08): ...

numerical_edge_match: Incomplete

@_dispatchable
def numerical_multiedge_match(attr, default, rtol: float = 1e-05, atol: float = 1e-08): ...
@_dispatchable
def generic_node_match(attr, default, op): ...

generic_edge_match: Incomplete

@_dispatchable
def generic_multiedge_match(attr, default, op): ...



from collections.abc import Callable, Sequence
from decimal import Decimal
from typing import Any, Literal, overload
from typing_extensions import Self

from wtforms.fields.core import Field, _Filter, _FormT, _Validator, _Widget
from wtforms.form import BaseForm
from wtforms.meta import DefaultMeta, _SupportsGettextAndNgettext
from wtforms.utils import UnsetValue

__all__ = ("IntegerField", "DecimalField", "FloatField", "IntegerRangeField", "DecimalRangeField")

class LocaleAwareNumberField(Field):
    use_locale: bool
    number_format: Any | None
    locale: str
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        use_locale: bool = False,
        # this accepts a babel.numbers.NumberPattern, but since it
        # is an optional dependency we don't want to depend on it
        # for annotating this one argument
        number_format: str | Any | None = None,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: object | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class IntegerField(Field):
    data: int | None
    # technically this is not as strict and will accept anything
    # that can be passed into int(), but we might as well be
    default: int | Callable[[], int] | None
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: int | Callable[[], int] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class DecimalField(LocaleAwareNumberField):
    data: Decimal | None
    # technically this is not as strict and will accept anything
    # that can be passed into Decimal(), but we might as well be
    default: Decimal | Callable[[], Decimal] | None
    places: int | None
    rounding: str | None
    @overload
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        *,
        places: UnsetValue = ...,
        rounding: None = None,
        use_locale: Literal[True],
        # this accepts a babel.numbers.NumberPattern, but since it
        # is an optional dependency we don't want to depend on it
        # for annotation this one argument
        number_format: str | Any | None = None,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: Decimal | Callable[[], Decimal] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...
    @overload
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        places: int | UnsetValue | None = ...,
        rounding: str | None = None,
        *,
        use_locale: Literal[False] = False,
        # this accepts a babel.numbers.NumberPattern, but since it
        # is an optional dependency we don't want to depend on it
        # for annotation this one argument
        number_format: str | Any | None = None,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: Decimal | Callable[[], Decimal] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class FloatField(Field):
    data: float | None
    # technically this is not as strict and will accept anything
    # that can be passed into float(), but we might as well be
    default: float | Callable[[], float] | None
    def __init__(
        self,
        label: str | None = None,
        validators: tuple[_Validator[_FormT, Self], ...] | list[Any] | None = None,
        *,
        filters: Sequence[_Filter] = (),
        description: str = "",
        id: str | None = None,
        default: float | Callable[[], float] | None = None,
        widget: _Widget[Self] | None = None,
        render_kw: dict[str, Any] | None = None,
        name: str | None = None,
        _form: BaseForm | None = None,
        _prefix: str = "",
        _translations: _SupportsGettextAndNgettext | None = None,
        _meta: DefaultMeta | None = None,
    ) -> None: ...

class IntegerRangeField(IntegerField): ...
class DecimalRangeField(DecimalField): ...



from braintree.merchant_account import MerchantAccount
from braintree.resource import Resource

class Merchant(Resource):
    merchant_accounts: list[MerchantAccount]
    def __init__(self, gateway, attributes) -> None: ...



from collections import defaultdict
from typing import Any, Generic, Literal, TypedDict, TypeVar, type_check_only

from yt_dlp.utils._jsruntime import BunJsRuntime, DenoJsRuntime, NodeJsRuntime, QuickJsRuntime

@type_check_only
class _SupportedJSRuntimes(TypedDict):
    deno: type[DenoJsRuntime]
    node: type[NodeJsRuntime]
    bun: type[BunJsRuntime]
    quickjs: type[QuickJsRuntime]

_T = TypeVar("_T")

class Indirect(Generic[_T]):
    value: _T
    def __init__(self, initial: _T, /) -> None: ...

postprocessors: Indirect[dict[str, Any]]
extractors: Indirect[dict[str, Any]]
all_plugins_loaded: Indirect[bool]
plugin_specs: Indirect[dict[str, Any]]
plugin_dirs: Indirect[list[str]]
plugin_ies: Indirect[dict[str, Any]]
plugin_pps: Indirect[dict[str, Any]]
plugin_ies_overrides: Indirect[defaultdict[str, Any]]
IN_CLI: Indirect[bool]
LAZY_EXTRACTORS: Indirect[bool | None]
WINDOWS_VT_MODE: Indirect[Literal[False] | None]  # Code takes into account that only False here
supported_js_runtimes: Indirect[_SupportedJSRuntimes]
supported_remote_components: Indirect[list[Literal["ejs:github", "ejs:npm"]]]



from collections.abc import Callable
from typing import ClassVar, Final
from typing_extensions import TypeAlias

from docutils import nodes
from docutils.parsers.rst import Directive

__docformat__: Final = "reStructuredText"

_DirectiveFn: TypeAlias = Callable[[str], str | list[str]]

class BasePseudoSection(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    node_class: ClassVar[type[nodes.Node] | None]
    invalid_parents: ClassVar[
        tuple[
            type[nodes.SubStructural],
            type[nodes.Bibliographic],
            type[nodes.Decorative],
            type[nodes.Body],
            type[nodes.Part],
            type[nodes.topic],
        ]
    ]
    def run(self): ...

class Topic(BasePseudoSection):
    node_class: ClassVar[type[nodes.Node]]

class Sidebar(BasePseudoSection):
    node_class: ClassVar[type[nodes.Node]]
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class LineBlock(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class ParsedLiteral(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class CodeBlock(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class MathBlock(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class Rubric(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class BlockQuote(Directive):
    classes: ClassVar[list[str]]
    def run(self): ...

class Epigraph(BlockQuote):
    classes: ClassVar[list[str]]

class Highlights(BlockQuote):
    classes: ClassVar[list[str]]

class PullQuote(BlockQuote):
    classes: ClassVar[list[str]]

class Compound(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...

class Container(Directive):
    option_spec: ClassVar[dict[str, _DirectiveFn]]
    def run(self): ...



from _typeshed import SliceableBuffer, Unused
from collections.abc import Sequence
from typing import Any, Protocol, type_check_only
from typing_extensions import TypeAlias

_FourIntSequence: TypeAlias = Sequence[int]
_TwoIntSequence: TypeAlias = Sequence[int]

@type_check_only
class _Kid(Protocol):
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "", /) -> str: ...

# All the classes below are used in version_file_info generated by `pyi-grab_version`
# See: https://pyinstaller.org/en/stable/usage.html#capturing-windows-version-data

# VSVersionInfo is also by other types referenced in https://pyinstaller.org/en/stable/spec-files.html#spec-file-operation
class VSVersionInfo:
    ffi: FixedFileInfo | None
    kids: list[_Kid]
    def __init__(self, ffi: FixedFileInfo | None = None, kids: list[_Kid] | None = None) -> None: ...
    def fromRaw(self, data: SliceableBuffer) -> int: ...
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "") -> str: ...

class FixedFileInfo:
    sig: int
    strucVersion: int
    fileVersionMS: int
    fileVersionLS: int
    productVersionMS: int
    productVersionLS: int
    fileFlagsMask: int
    fileFlags: int
    fileOS: int
    fileType: int
    fileSubtype: int
    fileDateMS: int
    fileDateLS: int
    def __init__(
        self,
        filevers: _FourIntSequence = (0, 0, 0, 0),
        prodvers: _FourIntSequence = (0, 0, 0, 0),
        mask: int = 0x3F,
        flags: int = 0x0,
        OS: int = 0x40004,
        fileType: int = 0x1,
        subtype: int = 0x0,
        date: _TwoIntSequence = (0, 0),
    ) -> None: ...
    def fromRaw(self, data: SliceableBuffer, i: int) -> int: ...
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "") -> str: ...

class StringFileInfo:
    name: str
    kids: list[_Kid]
    def __init__(self, kids: list[_Kid] | None = None) -> None: ...
    def fromRaw(self, sublen: Unused, vallen: Unused, name: str, data: SliceableBuffer, i: int, limit: int) -> int: ...
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "") -> str: ...

class StringTable:
    name: str
    kids: list[_Kid]
    def __init__(self, name: str | None = None, kids: list[_Kid] | None = None) -> None: ...
    def fromRaw(self, data: SliceableBuffer, i: int, limit: int) -> int: ...
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "") -> str: ...

class StringStruct:
    name: str
    val: str
    def __init__(self, name: str | None = None, val: str | None = None) -> None: ...
    def fromRaw(self, data: SliceableBuffer, i: int, limit: int) -> int: ...
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: Unused = "") -> str: ...

class VarFileInfo:
    kids: list[_Kid]
    def __init__(self, kids: list[_Kid] | None = None) -> None: ...
    sublen: int
    vallen: int
    name: str
    def fromRaw(self, sublen: int, vallen: int, name: str, data: SliceableBuffer, i: int, limit: int) -> int: ...
    wType: int
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: str = "") -> str: ...

class VarStruct:
    name: str
    kids: list[Any]  # Whatever can be passed to struct.pack
    def __init__(self, name: str | None = None, kids: list[Any] | None = None) -> None: ...
    def fromRaw(self, data: SliceableBuffer, i: int, limit: Unused) -> int: ...
    wValueLength: int
    wType: int
    sublen: int
    def toRaw(self) -> bytes: ...
    def __str__(self, indent: Unused = "") -> str: ...



import _tkinter
from _typeshed import StrPath
from typing import ClassVar

class ThemedWidget:
    pixmap_themes: ClassVar[list[str]]
    PACKAGES: ClassVar[dict[str, str]]
    tk: _tkinter.TkappType
    png_support: bool
    def __init__(self, tk_interpreter: _tkinter.TkappType, gif_override: bool = False) -> None: ...
    def set_theme(self, theme_name: str) -> None: ...
    def get_themes(self) -> list[str]: ...
    @property
    def themes(self) -> list[str]: ...
    @property
    def current_theme(self) -> str: ...
    def set_theme_advanced(
        self,
        theme_name: str,
        brightness: float = 1.0,
        saturation: float = 1.0,
        hue: float = 1.0,
        preserve_transparency: bool = True,
        output_dir: StrPath | None = None,
        advanced_name: str = "advanced",
    ) -> None: ...



from .australia import (
    Australia as Australia,
    AustralianCapitalTerritory as AustralianCapitalTerritory,
    Hobart as Hobart,
    NewSouthWales as NewSouthWales,
    NorthernTerritory as NorthernTerritory,
    Queensland as Queensland,
    SouthAustralia as SouthAustralia,
    Tasmania as Tasmania,
    Victoria as Victoria,
    WesternAustralia as WesternAustralia,
)
from .marshall_islands import MarshallIslands as MarshallIslands
from .new_zealand import NewZealand as NewZealand

__all__ = (
    # Australia and al.
    "Australia",
    "AustralianCapitalTerritory",
    "NewSouthWales",
    "NorthernTerritory",
    "Queensland",
    "SouthAustralia",
    "Tasmania",
    "Hobart",
    "Victoria",
    "WesternAustralia",
    # Other oceanian countries
    "MarshallIslands",
    "NewZealand",
)



from _typeshed import Incomplete
from typing import IO, Literal

from reportlab.lib.colors import Color, _ConvertibleToColor
from reportlab.pdfgen.textobject import PDFTextObject, _PDFColorSetter

class ShowBoundaryValue:
    color: _ConvertibleToColor | None
    width: float
    dashArray: Incomplete
    def __init__(
        self,
        color: _ConvertibleToColor | None = (0, 0, 0),
        width: float = 0.1,
        dashArray: list[float] | tuple[float, ...] | None = None,
    ) -> None: ...
    def __bool__(self) -> bool: ...

class Canvas(_PDFColorSetter):
    bottomup: int
    imageCaching: Incomplete
    state_stack: Incomplete
    def __init__(
        self,
        filename: str | IO[bytes],
        pagesize: tuple[float, float] | None = None,
        bottomup: int = 1,
        pageCompression=None,
        invariant=None,
        verbosity: int = 0,
        encrypt=None,
        cropMarks=None,
        pdfVersion=None,
        enforceColorSpace=None,
        initialFontName: float | None = None,
        initialFontSize: float | None = None,
        initialLeading: float | None = None,
        cropBox=None,
        artBox=None,
        trimBox=None,
        bleedBox=None,
        lang=None,
    ) -> None: ...
    def setEncrypt(self, encrypt) -> None: ...
    def init_graphics_state(self) -> None: ...
    def push_state_stack(self) -> None: ...
    def pop_state_stack(self) -> None: ...
    STATE_ATTRIBUTES: Incomplete
    STATE_RANGE: Incomplete
    def setAuthor(self, author: str | None) -> None: ...
    def setDateFormatter(self, dateFormatter) -> None: ...
    def addOutlineEntry(self, title, key, level: int = 0, closed=None) -> None: ...
    def setOutlineNames0(self, *nametree) -> None: ...
    def setTitle(self, title: str | None) -> None: ...
    def setSubject(self, subject: str | None) -> None: ...
    def setCreator(self, creator: str | None) -> None: ...
    def setProducer(self, producer: str | None) -> None: ...
    def setKeywords(self, keywords: str | None) -> None: ...
    def pageHasData(self): ...
    def showOutline(self) -> None: ...
    def showFullScreen0(self) -> None: ...
    def setBlendMode(self, v) -> None: ...
    def showPage(self) -> None: ...
    def setPageCallBack(self, func) -> None: ...
    def bookmarkPage(self, key, fit: str = "Fit", left=None, top=None, bottom=None, right=None, zoom=None): ...
    def bookmarkHorizontalAbsolute(self, key, top, left: int = 0, fit: str = "XYZ", **kw): ...
    def bookmarkHorizontal(self, key, relativeX, relativeY, **kw) -> None: ...
    def doForm(self, name) -> None: ...
    def hasForm(self, name): ...
    def drawInlineImage(
        self,
        image,
        x: float,
        y: float,
        width: float | None = None,
        height: float | None = None,
        preserveAspectRatio: bool = False,
        anchor: str = "c",
        anchorAtXY: bool = False,
        showBoundary: bool = False,
        extraReturn=None,
    ): ...
    def drawImage(
        self,
        image,
        x: float,
        y: float,
        width: float | None = None,
        height: float | None = None,
        mask=None,
        preserveAspectRatio: bool = False,
        anchor: str = "c",
        anchorAtXY: bool = False,
        showBoundary: bool = False,
        extraReturn=None,
    ): ...
    def beginForm(self, name, lowerx: int = 0, lowery: int = 0, upperx=None, uppery=None) -> None: ...
    def endForm(self, **extra_attributes) -> None: ...
    def addPostScriptCommand(self, command, position: int = 1) -> None: ...
    def freeTextAnnotation(self, contents, DA, Rect=None, addtopage: int = 1, name=None, relative: int = 0, **kw) -> None: ...
    def textAnnotation(self, contents, Rect=None, addtopage: int = 1, name=None, relative: int = 0, **kw) -> None: ...
    textAnnotation0 = textAnnotation
    def highlightAnnotation(
        self, contents, Rect, QuadPoints=None, Color=[0.83, 0.89, 0.95], addtopage: int = 1, name=None, relative: int = 0, **kw
    ) -> None: ...
    def inkAnnotation(
        self, contents, InkList=None, Rect=None, addtopage: int = 1, name=None, relative: int = 0, **kw
    ) -> None: ...
    inkAnnotation0 = inkAnnotation
    def linkAbsolute(
        self,
        contents,
        destinationname,
        Rect=None,
        addtopage: int = 1,
        name=None,
        thickness: int = 0,
        color: Color | None = None,
        dashArray=None,
        **kw,
    ): ...
    def linkRect(
        self,
        contents,
        destinationname,
        Rect=None,
        addtopage: int = 1,
        name=None,
        relative: int = 1,
        thickness: int = 0,
        color: Color | None = None,
        dashArray=None,
        **kw,
    ): ...
    def linkURL(
        self,
        url,
        rect,
        relative: int = 0,
        thickness: int = 0,
        color: Color | None = None,
        dashArray=None,
        kind: str = "URI",
        **kw,
    ) -> None: ...
    def getPageNumber(self) -> int: ...
    def save(self) -> None: ...
    def getpdfdata(self): ...
    def setPageSize(self, size: tuple[float, float]) -> None: ...
    def setCropBox(self, size, name: str = "crop") -> None: ...
    def setTrimBox(self, size) -> None: ...
    def setArtBox(self, size) -> None: ...
    def setBleedBox(self, size) -> None: ...
    # NOTE: Only accepts right angles
    def setPageRotation(self, rot: float) -> None: ...
    def addLiteral(self, s: object, escaped: int = 1) -> None: ...
    def resetTransforms(self) -> None: ...
    def transform(self, a: float, b: float, c: float, d: float, e: float, f: float) -> None: ...
    def absolutePosition(self, x: float, y: float) -> tuple[float, float]: ...
    def translate(self, dx: float, dy: float) -> None: ...
    def scale(self, x: float, y: float) -> None: ...
    def rotate(self, theta: float) -> None: ...
    def skew(self, alpha: float, beta: float) -> None: ...
    def saveState(self) -> None: ...
    def restoreState(self) -> None: ...
    def line(self, x1: float, y1: float, x2: float, y2: float) -> None: ...
    def lines(self, linelist) -> None: ...
    def cross(
        self,
        x: float,
        y: float,
        size: float = 5,
        gap: float = 1,
        text=None,
        strokeColor=None,
        strokeWidth: float | None = None,
        fontSize: float = 3,
    ) -> None: ...
    def grid(self, xlist, ylist) -> None: ...
    def bezier(self, x1: float, y1: float, x2: float, y2: float, x3: float, y3: float, x4: float, y4: float) -> None: ...
    def arc(self, x1: float, y1: float, x2: float, y2: float, startAng: float = 0, extent: float = 90) -> None: ...
    def rect(self, x: float, y: float, width: float, height: float, stroke: float = 1, fill: float = 0) -> None: ...
    def ellipse(self, x1: float, y1: float, x2: float, y2: float, stroke: float = 1, fill: float = 0) -> None: ...
    def wedge(
        self, x1: float, y1: float, x2: float, y2: float, startAng: float, extent: float, stroke: float = 1, fill: float = 0
    ) -> None: ...
    def circle(self, x_cen: float, y_cen: float, r: float, stroke: float = 1, fill: float = 0) -> None: ...
    def roundRect(
        self, x: float, y: float, width: float, height: float, radius: float, stroke: float = 1, fill: float = 0
    ) -> None: ...
    def shade(self, shading) -> None: ...
    def linearGradient(self, x0: float, y0: float, x1: float, y1: float, colors, positions=None, extend: bool = True) -> None: ...
    def radialGradient(self, x: float, y: float, radius: float, colors, positions=None, extend: bool = True) -> None: ...
    def drawString(
        self,
        x: float,
        y: float,
        text: str,
        mode: Literal[0, 1, 2, 3, 4, 5, 6, 7] | None = None,
        charSpace: float = 0,
        direction: Literal["LTR", "RTL"] | None = None,
        wordSpace: float | None = None,
        shaping: bool = False,
    ) -> None: ...
    def drawRightString(
        self,
        x: float,
        y: float,
        text: str,
        mode: Literal[0, 1, 2, 3, 4, 5, 6, 7] | None = None,
        charSpace: float = 0,
        direction: Literal["LTR", "RTL"] | None = None,
        wordSpace: float | None = None,
        shaping: bool = False,
    ) -> None: ...
    def drawCentredString(
        self,
        x: float,
        y: float,
        text: str,
        mode: Literal[0, 1, 2, 3, 4, 5, 6, 7] | None = None,
        charSpace: float = 0,
        direction: Literal["LTR", "RTL"] | None = None,
        wordSpace: float | None = None,
        shaping: bool = False,
    ) -> None: ...
    def drawAlignedString(
        self,
        x: float,
        y: float,
        text: str,
        pivotChar: str = ".",
        mode: Literal[0, 1, 2, 3, 4, 5, 6, 7] | None = None,
        charSpace: float = 0,
        direction: Literal["LTR", "RTL"] | None = None,
        wordSpace: float | None = None,
        shaping: bool = False,
    ) -> None: ...
    def getAvailableFonts(self): ...
    def listLoadedFonts0(self): ...
    def setFont(self, psfontname: str, size: float, leading: float | None = None) -> None: ...
    def setFontSize(self, size: float | None = None, leading: float | None = None) -> None: ...
    def stringWidth(self, text: str, fontName: str | None = None, fontSize: float | None = None) -> float: ...
    def setLineWidth(self, width: float) -> None: ...
    def setLineCap(self, mode) -> None: ...
    def setLineJoin(self, mode) -> None: ...
    def setMiterLimit(self, limit) -> None: ...
    def setDash(self, array: list[float] | tuple[float, ...] | float = [], phase: float = 0) -> None: ...
    def beginPath(self): ...
    def drawPath(self, aPath, stroke: int = 1, fill: int = 0, fillMode=None) -> None: ...
    def clipPath(self, aPath, stroke: int = 1, fill: int = 0, fillMode=None) -> None: ...
    def beginText(self, x: float = 0, y: float = 0, direction: Literal["LTR", "RTL"] | None = None) -> PDFTextObject: ...
    def drawText(self, aTextObject: PDFTextObject) -> None: ...
    def setPageCompression(self, pageCompression: int = 1) -> None: ...
    def setPageDuration(self, duration=None) -> None: ...
    def setPageTransition(
        self, effectname: str | None = None, duration: float = 1, direction: float = 0, dimension: str = "H", motion: str = "I"
    ) -> None: ...
    def getCurrentPageContent(self): ...
    def setViewerPreference(self, pref, value) -> None: ...
    def getViewerPreference(self, pref): ...
    def delViewerPreference(self, pref) -> None: ...
    def setCatalogEntry(self, key, value) -> None: ...
    def getCatalogEntry(self, key): ...
    def delCatalogEntry(self, key) -> None: ...
    def addPageLabel(self, pageNum, style=None, start=None, prefix=None) -> None: ...
    @property
    def acroForm(self): ...
    def drawBoundary(self, sb, x1: float, y1: float, width: float, height: float) -> None: ...

__all__ = ["Canvas", "ShowBoundaryValue"]
